\\ Copyright 2015, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


allocatemem(200*10^6);
default(recover,0);
read("dragon-defines.gp");

\\-----------------------------------------------------------------------------

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
points_to_centroid(v) =
{
  my(centroid=geom_centroid_of_points(v));
  apply(z->z-centroid,v);
}

RQ_0_points = [ 1/2 - 1/2*I ];
IRQ_0_at_start = geom_inertia_of_points(RQ_0_points) /* untransformed */
RQ_0_points = points_to_centroid(RQ_0_points);
RQ_0_points = geom_points_rotate_xydir(RQ_0_points, conj(b)^0)
IRQ_0 = geom_inertia_of_points(RQ_0_points)

RQ_1_points = [ 1/2 - 1/2*I, 3/2 + 1/2*I ];
check_exact(GRQ(1), geom_centroid_of_points(RQ_1_points));
IRQ_1_at_start = geom_inertia_of_points(RQ_1_points); /* untransformed */
RQ_1_points = points_to_centroid(RQ_1_points);
check_exact(geom_inertia_of_points(RQ_1_points) \
            + #RQ_1_points * geom_inertia_of_point(GRQ(1)),  \
            IRQ_1_at_start, \
            "IRQ_1_at_start");
RQ_1_points = geom_points_rotate_xydir(RQ_1_points, conj(b)^1)
IRQ_1 = geom_inertia_of_points(RQ_1_points);

GRQ(2)
RQ_2_points = points_to_centroid({[ 1/2 - 1/2*I,
                                   3/2 + 1/2*I,
                                   1/2 + 3/2*I ]});
RQ_2_points = geom_points_rotate_xydir(RQ_2_points, conj(b)^2)
IRQ_2 = geom_inertia_of_points(RQ_2_points)

\\          @
\\    *   *---*
\\    | @ |   | @
\\    *---*  G*---*
\\                | @        
\\            *---*
\\              @      
check_exact(GRQ(3), 1/10 + 11/10*I);
RQ_3_points = {[ 1/2 - 1/2*I, 3/2 + 1/2*I, 1/2 + 3/2*I,
                                   -1/2 + (2+1/2)*I,  -3/2 + (1+1/2)*I
                                  ]};
check_exact(geom_centroid_of_points(RQ_3_points),GRQ(3));
RQ_3_points=points_to_centroid(RQ_3_points);
RQ_3_points = geom_points_rotate_xydir(RQ_3_points, conj(b)^3)
print("IRQ_3");
IRQ_3 = geom_inertia_of_points(RQ_3_points)


\\           @       @
\\         *---*   *---*
\\       @ |   | @ |   | @
\\         *---*---*  G*---*
\\           @ |           | @        
\\     *   *---*       *---*
\\     | @ |             @      
\\     *---*
check_exact(RQ(4), 9);
RQ_4_points = {[ 1/2 - 1/2*I, 3/2 + 1/2*I, 1/2 + 3/2*I,
                -1/2 + (2+1/2)*I,  -3/2 + (1+1/2)*I,
                -(2+1/2) + (2+1/2)*I,
                -(3+1/2) + (1+1/2)*I,
                -(2+1/2) + 1/2*I,
                -(3+1/2) - 1/2*I]};
IRQ_4_at_start = geom_inertia_of_points(RQ_4_points); /* untransformed */
check_exact(GRQ(4), -23/18 + 19/18*I, "GRQ(4)");
check_exact(geom_centroid_of_points(RQ_4_points),GRQ(4));
RQ_4_points=points_to_centroid(RQ_4_points);
RQ_4_points = geom_points_rotate_xydir(RQ_4_points, conj(b)^4)
print("IRQ_4");
IRQ_4 = geom_inertia_of_points(RQ_4_points);


\\-----------------------------------------------------------------------------

G_to_start(k) = - GRQ(k);
G_to_Gr(k) = G_to_start(k) + GRQ(k-1);
G_to_Gr1(k) = G_to_start(k) + b^(k-1) + (-1)*GRQ(k-3);
G_to_Gr2(k) = G_to_start(k) + b^k  + GRQ(k-3);
Gr(k)  = GRQ(k) + G_to_Gr(k);
Gr1(k) = GRQ(k) + G_to_Gr1(k);
Gr2(k) = GRQ(k) + G_to_Gr2(k);

my(k=3); check_exact(b^(k-1), 2*I);
check_exact(GRQ(0), 1/2 - 1/2*I);
check_exact(I*GRQ(0), 1/2 + 1/2*I);
check_exact(GRQ(3), 1/10 + 11/10*I);
check_exact(Gr1(3), -1/2 + (2+1/2)*I, "Gr1(3)");
check_exact(Gr2(3), -3/2 + (1+1/2)*I, "Gr2(3)");
check_exact(Gr(3),  GRQ(2), "Gr(3)");

\\              / \                   *----*
\\  *          /    *                 |
\\    \   G    \  v  IRQ              |    v   k-3
\\  IRQ \      ^                      *----*
\\        v  / IRQ                   G     ^
\\         *                               |   k-3
\\                              *--------->*
\\                                  k-1

IRQ(k) =
{
  \\ print("IRQ("k") called");
  if(k<0, error("IRQ() is for k>=0, got ",k));
  if(k==0, return(IRQ_0));
  if(k==1, return(IRQ_1));
  if(k==2, return(IRQ_2));

  my(xydir = conj(b)^k);
        R45^-1 * IRQ(k-1) * R45             \\ axis +45
  + 2 * R45    * IRQ(k-3) * R45^-1          \\ axis -45
  + 
  RQ(k-1) * 
  geom_inertia_of_point(geom_point_rotate_xydir(G_to_Gr(k), xydir))
  + 
  RQ(k-3) * 
  geom_inertia_of_point(geom_point_rotate_xydir(G_to_Gr1(k), xydir))
  + 
  RQ(k-3) * 
  geom_inertia_of_point(geom_point_rotate_xydir(G_to_Gr2(k), xydir))
  ;
}
IRQ=memoize(IRQ);

check_exact(IRQ(3), IRQ_3, "IRQ(3)");
check_exact(IRQ(4), IRQ_4, "IRQ(4)");

check_exact(GRQ(8), 753/146 - 653/146*I);
print("IRQ 8");
IRQ(8)

\\ unrotated and relative to origin 0
IRQ_at_start(k) =
{
  geom_inertia_rotate_xydir(IRQ(k), conj(b)^k)
  + RQ(k) * geom_inertia_of_point(GRQ(k));
}
check_exact(IRQ_at_start(0), IRQ_0_at_start, "IRQ_at_start(0)");
check_exact(IRQ_at_start(1), IRQ_1_at_start, "IRQ_at_start(1)");
check_exact(IRQ_at_start(4), IRQ_4_at_start, "IRQ_at_start(4)");

\\ print("IRQ_at_start 8");
\\ IRQ_at_start(8)
\\ values from dragon.pl
check_exact(IRQ_at_start(8), \
            geom_inertia_of_Ix_Iy_Ixy(10705/4, 20937/4, -6453/4), \
            "IRQ_at_start(8)");

\\ IRQ_part_1_only(k) =
\\ {
\\   my(xydir = conj(b)^k);
\\   R45    * IRQ(k-1) * R45^-1
\\   +
\\   RQ(k-3) * 
\\   geom_inertia_of_point(geom_point_rotate_xydir(G_to_Gr1(k), xydir));
\\ }
\\ IRQ_part_1_only(3)
\\ 
\\ {
\\   my(z = -1/2 + (2+1/2)*I);
\\   z -= GRQ(3);
\\   z = geom_point_rotate_xydir(z, conj(b)^3);
\\   geom_inertia_of_point(z)
\\ }

\\ {
\\   my(v=vector(1000,k,k--; RQ(k)*geom_inertia_Ix(IRQ(k))));
\\   recurrence_guess(v);
\\ }

gIRQx_times_RQ(x) =
{
  -45/164                        / (1 - x)
  + (351/1681 - 1197/6724*x)     / (1 + x^2)

  + (-479/20938 + 1379/10469*x - 117/1102*x^2)
  / (1 + 2*x^2 - 4*x^3)

  + (4481/20938 + 8595/20938*x + 12653/41876*x^2)
  /(1 - x - 4*x^2 - 4*x^3)

  + (-26453577/698910440 - 236283643/7163832010*x + 756530141/3581916005*x^2)
  /(1 + 8*x^2 - 32*x^3)

  \\---------
  + (6247/1936040 - 21611/242005*x - 9023/484010*x^2 + 22088/48401*x^3 - 3211/96802*x^4 + 162378/242005*x^5)
  /(1 - 4*x^2 + 4*x^4 + 16*x^6)
  
  + (-82981/1936040 + 96344/242005*x - 60953/1936040*x^2 + 43527/193604*x^3 - 8099/96802*x^4 + 163437/242005*x^5)
  /(1 + 9*x^2 + 8*x^4 + 16*x^6)
  
  + (-210/1681 - 1257/1681*x - 1284/1681*x^2 - 702/1681*x^3 - 5244/1681*x^4 - 4464/1681*x^5)
  /(1 - 2*x + 2*x^2 + 8*x^3 + 32*x^6)

  \\---------
  \\ 2^k*RQ(k)^2 is (1 - 2*x - 16*x^2 - 32*x^3)

  + (90300525081/1174868449640 + 155230676323/587434224820*x + 50206759214/146858556205*x^2)
  /(1 - 2*x - 16*x^2 - 32*x^3)
  ;
}
check_exact(gf_terms(gIRQx_times_RQ(x),100), \
            vector(100,k,k--; RQ(k)*geom_inertia_Ix(IRQ(k))), \
            "gIRQx_times_RQ");


\\ polroots(x^3*subst(1 + 8*x^2 - 32*x^3, 'x,1/'x))
\\ 2*r*(r-1)
\\ cubic_r2*2*r
\\ cubic_r3*2*r
\\ r*(r-1)

\\ 1 - 2*x + x^2 - 4*x^3 + 4*x^4 + 4*x^6
pol_ascending_print((1-x-2*x^3)^2);
pol_ascending_print((1-x-2*x^3)^2);

\\ 1/(1 - 2*x - 16*x^2 - 32*x^3)
\\ (x^3 - 2*x^2 - 16*x - 32)
\\ 5.750259588325557651947941189
\\ -1.875129794162778825973970594 - 1.431381993571929481142030080*I,
\\ -1.875129794162778825973970594 + 1.431381993571929481142030080*I
\\ 2*r^2
\\ 2*cubic_r2^2
\\ 2*cubic_r3^2

\\ (x-2*r^2)*(x-2*cubic_r2^2)*(x-2*cubic_r3^2)
\\ pol_ascending_print((1-2*r^2*x)*(1-2*cubic_r2^2*x)*(1-2*cubic_r3^2*x))


\\ (-1.875129794162778825973970594 - 1.431381993571929481142030080*I)
\\ (x-r)^2
\\ -0.6956207695598620574163671003 + 2.057704508273384806866166016*I
\\ 1-r \\ Re
\\ real(cubic_r2)*2  \\ Im
\\ imag(cubic_r2)*2  \\ Im


\\ Ix*RQ(k) grows as 2^k*(r^2)^k
\\
p=x^33 - 6*x^32 + 11*x^31 - 96*x^30 + 235*x^29 - 150*x^28 + 1081*x^27 - 1124*x^26 + 1592*x^25 - 16904*x^24 + 49264*x^23 + 22688*x^22 - 117248*x^21 - 325312*x^20 - 87632*x^19 + 174272*x^18 + 2012736*x^17 - 2027008*x^16 + 2738432*x^15 + 1301504*x^14 - 3951616*x^13 - 25378816*x^12 - 4640768*x^11 - 35667968*x^10 + 21544960*x^9 - 16646144*x^8 + 77791232*x^7 - 39583744*x^6 + 129761280*x^5 - 4194304*x^4 + 106954752*x^3 - 109051904*x^2 + 33554432*x - 134217728;
print("p max root / r^2", vecmax(abs(polroots(p))) /r^2);

\\ recurrence_guess(vector(200,k,k--; 2^k*RQ(k)^2));
\\ recurrence_guess(vector(200,k,k--; 2^k*RQ(k)*RQ(k+1)));
\\ recurrence_guess(vector(200,k,k--; 2^k*RQ(k)*RQ(k+2)));

\\ recurrence_guess(vector(1000,k,k--; LQ(k)^2));
gLQ_squared(x) =
{
  (25/58 - 17/29*x + 11/29*x^2)   / (1 + 2*x^2 - 4*x^3)
  + (33/58 + 59/58*x + 18/29*x^2) / (1 - x - 4*x^2 - 4*x^3);
}
check_exact(gf_terms(gLQ_squared(x),100), vector(100,k,k--; LQ(k)^2));

\\ recurrence_guess(vector(1000,k,k--; RQ(k)^2));
gRQ_squared(x) =
{
  (-7/58 + 21/29*x - 17/29*x^2)  / (1 + 2*x^2 - 4*x^3)
  + (65/58 + 125/58*x + 46/29*x^2) / (1 - x - 4*x^2 - 4*x^3);
}
check_exact(gf_terms(gRQ_squared(x),100), vector(100,k,k--; RQ(k)^2), \
            "gRQ_squared");

\\recurrence_guess(vector(1000,k,k--; 2^k*RQ(k)^2));
gRQ_squared_times_2pow(x) =
{
  (-7/58 + 42/29*x - 68/29*x^2)/(1 + 8*x^2 - 32*x^3)
  + (65/58 + 125/29*x + 184/29*x^2)/(1 - 2*x - 16*x^2 - 32*x^3);
}
check_exact(gf_terms(gRQ_squared_times_2pow(x),100), \
            vector(100,k,k--; 2^k*RQ(k)^2));

\\recurrence_guess(vector(1000,k,k--; 2^k*RQ(k)*RQ(k+1)));
gRQ_times_RQ1_times_2pow(x) =
{
  (5/58 + 28/29*x - 84/29*x^2)/(1 + 8*x^2 - 32*x^3)
  + (111/58 + 209/29*x + 316/29*x^2)/(1 - 2*x - 16*x^2 - 32*x^3);
}
check_exact(gf_terms(gRQ_times_RQ1_times_2pow(x),100), \
            vector(100,k,k--; 2^k*RQ(k)*RQ(k+1)));

\\recurrence_guess(vector(1000,k,k--; 2^k*RQ(k)*RQ(k+2)));
gRQ_times_RQ2_times_2pow(x) =
{
  (-8/29 + 38/29*x - 56/29*x^2)/(1 + 8*x^2 - 32*x^3)
  + (95/29 + 352/29*x + 520/29*x^2)/(1 - 2*x - 16*x^2 - 32*x^3);
}
check_exact(gf_terms(gRQ_times_RQ2_times_2pow(x),100), \
            vector(100,k,k--; 2^k*RQ(k)*RQ(k+2)));

{
  lindep([(90300525081/1174868449640 + 155230676323/587434224820*x + 50206759214/146858556205*x^2)
          /(1 - 2*x - 16*x^2 - 32*x^3),

          (65/58 + 125/29*x + 184/29*x^2)/(1 - 2*x - 16*x^2 - 32*x^3),
          (111/58 + 209/29*x + 316/29*x^2)/(1 - 2*x - 16*x^2 - 32*x^3),
          (95/29 + 352/29*x + 520/29*x^2)/(1 - 2*x - 16*x^2 - 32*x^3) ]
         * (1 - 2*x - 16*x^2 - 32*x^3))
}


\\ \\ gf_terms(gIRQx(x),10)
\\ vector(10,k,k--; RQ(k)*geom_inertia_Ix(IRQ(k)))
\\ vector(10,k,k--; RQ(k))
\\ vector(10,k,k--; geom_inertia_Ix(IRQ(k)))
\\ \\recurrence_guess(vector(200,k,k--; floor(simplify(geom_inertia_Ix(IRQ(k))))))
\\ my(k=100); geom_inertia_principal_xyangle(simplify(IRQ(k)))*180/Pi
\\ 
\\ \\ [0.06456998503147040724584228121 -0.005041080855732327547392348438 0    ]
\\ \\ [-0.005041080855732327547392348438 0.1758010817718460203671905033 0    ]
\\ \\ [0     0     0.2403710668033164276130327845]
\\ my(k=100); IRQ(k)/2^k/RQ(k)*1.0
