\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


allocatemem(100*10^6);


read("../tools/memoize.gp");

b = 1+I;
L(k) = if(k<0,error("L k<0"), k<=3,2^k, L(k-1) + 2*L(k-3));
L=memoize(L);
R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);

\\-----------------------------------------------------------------------------

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_equal(got,want) =
{
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok"));
}

check_exact(got,want) =
{
  if(got!= want,
     print("wrong\n"); quit(1),
     print("ok"));
}

\\-----------------------------------------------------------------------------

\\                               1           L[k+1] = T[k]
\\                               |           R[k+1] = R[k] + L[k]
\\                            T  | L
\\        L                      v           initial R[0] = 1
\\     0 ---> 1           0 ---> *                   L[0] = 1
\\        R                  R
\\
dragon_left_boundary_midpoints =
{
  memoize(k->
          \\ print("L ",k);
          if(k<0, error("not k<0"));
          if(k==0, return([1/2]));
          dragon_left_boundary_midpoints_T(k-1));
}

\\ T expands to a U and an R.
\\
\\                           2
\\                           |
\\                         R |
\\                           v
\\             2             * <--- 1        T[k+1] = U[k] + R[k]
\\             |                    |
\\          T  |                 U  |        initial
\\             v                    v        T[0] = 2
\\      0 ---> 1             0 ---> *
\\
dragon_left_boundary_midpoints_T =
{
  memoize(k->
          \\ print("T ",k);
          if(k<0, error("not k<0"));
          if(k==0, return([1/2, 1+1/2*I]));
          my(end=2*I*b^(k-1));
          concat(dragon_left_boundary_midpoints_U(k-1),
                 apply(z->end - I*z,
                      Vecrev(dragon_right_boundary_midpoints(k-1)))));
}

\\ U expands to a U and a V.
\\
\\                      * <--- 2
\\                      ^      |
\\                      |  V   |
\\                      |      v
\\      3 <--- 2        3      * <--- 1      U[k+1] = U[k] + V[k]
\\             |                      |
\\          U  |                   U  |      initial
\\             v                      v      U[0] = 3
\\      0 ---> 1               0 ---> *
\\
dragon_left_boundary_midpoints_U =
{
  memoize(k->
          \\ print("U ",k);
          if(k<0, error("not k<0"));
          if(k==0, return([1/2, 1+1/2*I, 1/2+I]));
          my(end=I*b^(k-1));
          concat(dragon_left_boundary_midpoints_U(k-1),
                 apply(z->end + I*z,
                      dragon_left_boundary_midpoints_V(k-1))));
}

\\ V expands to a T because the curve touches itself and so closes off the "U"
\\ shape.  The effect is that the V values are V[0]=3 thereafter the T values
\\ V[1]=T[0], V[2]=T[1], etc.
\\
\\                             5 <--- *
\\                             |      |
\\                             |      |
\\                             v      v
\\      6 <--- 5        6 ---> * ---> 4      V[k+1] = T[k]
\\             |               ^
\\          V  |            T  |             initial V[0]=3
\\             v               |
\\      3 ---> 4               3
\\
dragon_left_boundary_midpoints_V =
{
  memoize(k->
          \\ print("V ",k);
          if(k<0, error("not k<0"));
          if(k==0, return([1/2, 1+1/2*I, 1/2+I]));
          I*dragon_left_boundary_midpoints(k));
}

\\              1
\\              |
\\              |  L
\\              v
\\       0 ---> *
\\          R
\\
dragon_right_boundary_midpoints =
{
  memoize(k->
          \\ print("R ",k);
          if(k==0, return([1/2]));
          my(end=b^k);
          concat(dragon_right_boundary_midpoints(k-1),
                 apply(z->end - I*z,
                      Vecrev(dragon_left_boundary_midpoints(k-1)))));
}

\\ L
check_equal(dragon_left_boundary_midpoints(0), [1/2])
check_equal(dragon_left_boundary_midpoints(1), [1/2, 1+1/2*I])
check_equal(dragon_left_boundary_midpoints(2), [1/2, 1+1/2*I, 1/2+I, 3/2*I])
check_equal(dragon_left_boundary_midpoints(3), \
            [1/2, 1+1/2*I, 1/2+I, 3/2*I,  -1/2+2*I, -1+3/2*I, -3/2+I, -2+3/2*I])
check_equal(dragon_left_boundary_midpoints(4), \
            [1/2, 1+1/2*I, 1/2+I, 3/2*I,  -1/2+2*I, -1+3/2*I, -3/2+I, \
             -2+1/2*I, -5/2, -3-1/2*I, -7/2-I, -4-1/2*I])

\\ T
check_equal(dragon_left_boundary_midpoints_T(0), [1/2, 1+1/2*I])
check_equal(dragon_left_boundary_midpoints_T(1), [1/2, 1+1/2*I, 1/2+I, 3/2*I])

\\ U
check_equal(dragon_left_boundary_midpoints_U(0), [1/2, 1+1/2*I, 1/2+I])
check_equal(dragon_left_boundary_midpoints_U(1), [1/2, 1+1/2*I, 1/2+I, \
                                                  3/2*I, -1/2+2*I, -1+3/2*I])
\\check_equal(dragon_left_boundary_midpoints_U(1), [1/2, 1+1/2*I, 1/2+I])

\\ V
check_equal(dragon_left_boundary_midpoints_V(0), [1/2, 1+1/2*I, 1/2+I])
check_equal(dragon_left_boundary_midpoints_V(1), [1/2*I, -1/2+I])

\\ R
check_equal(dragon_right_boundary_midpoints(0), [1/2])
check_equal(dragon_right_boundary_midpoints(1), [1/2, 1+1/2*I])
check_equal(dragon_right_boundary_midpoints(2), [1/2, 1+1/2*I, 1/2+I, 3/2*I])
check_equal(dragon_right_boundary_midpoints(3), \
            [1/2, 1+1/2*I, 1/2+I, 3/2*I,  -1/2+2*I, -1+3/2*I, -3/2+I, -2+3/2*I])

\\-----------------------------------------------------------------------------

left_boundary_centroid_by_points(k) = vecsum(dragon_left_boundary_midpoints(k));
right_boundary_centroid_by_points(k) = vecsum(dragon_right_boundary_midpoints(k));

print("initial GL=",vector(4,k,left_boundary_centroid_by_points(k-1)));

print("GLN samples");
vector(20,k,my(k=k-1); left_boundary_centroid_by_points(k))


\\               R              R
\\           * <--- 4       * <--- 2
\\           ^      |       ^      |
\\         L |      |   U   |   V  |  T
\\           |      v       |      v
\\           5 ---> * <---- 3      * <--- 1
\\                  ^                     |
\\               V  |  T              U   |  L
\\                  |                     v
\\   8       * <--- 6              0 ---> *
\\   |       ^                        R
\\ R |   U   |  T
\\   v       |
\\   * <---  7
\\       L
\\

left_midpoints_by_breakdown(k) =
{
  if(k<4,return(dragon_left_boundary_midpoints(k)));

  my(v1  = dragon_left_boundary_midpoints(k-1),
     v3  = dragon_left_boundary_midpoints(k-3),
     r   = R(k-3),
     v1a = v1[1 .. -r-1],
     v1b = v1[-r .. #v1],
     bk  = b^(k-4),
     p1  = bk*(-1+I),
     p2  = bk*-2,
     offset = bk*(-2-2*I));
  \\ print("v1a ",v1a);
  \\ print("v1b ",v1b);
  \\ print("v1b offset ",apply(z->offset+z, v1b));
  \\ print("v3 first ", apply(z->p1-z, v3));
  concat([v1a,
          apply(z->p1-z, v3),
          apply(z->p2-z, v3),
          apply(z->offset+z, v1b)]);
}
check_equal(L(4), L(3)+2*L(1))
check_equal(left_midpoints_by_breakdown(4), \
            [1/2, 1+1/2*I, 1/2+I, 3/2*I,  -1/2+2*I, -1+3/2*I, -3/2+I, \
             -2+1/2*I, -5/2, -3-1/2*I, -7/2-I, -4-1/2*I])

left_boundary_centroid =
{
  memoize(k->
          if(k<4, return(left_boundary_centroid_by_points(k)));
          my(bk = b^(k-4));
          left_boundary_centroid(k-1)
          + -1*left_boundary_centroid(k-3)  +  L(k-3)*bk*(-1+I)
          + -1*left_boundary_centroid(k-3)  +  L(k-3)*bk*(-2)
          + R(k-3)*bk*(-2-2*I));
}
check_equal(vector(10,k,my(k=k-1); left_boundary_centroid_by_points(k)), \
            vector(10,k,my(k=k-1); left_boundary_centroid(k)))

left_boundary_centroid =
{
  memoize(k->
          if(k<4, return(left_boundary_centroid_by_points(k)));
          my(bk = b^(k-4));
          left_boundary_centroid(k-1)
          - 2*left_boundary_centroid(k-3)
          + L(k-3)*bk*(-3+I)
          + R(k-3)*bk*(-2-2*I));
}
check_equal(vector(10,k,my(k=k-1); left_boundary_centroid_by_points(k)), \
            vector(10,k,my(k=k-1); left_boundary_centroid(k)))


left_boundary_centroid =
{
  memoize(k->
          if(k<4, return(left_boundary_centroid_by_points(k)));
          my(bk = b^(k-4));
          left_boundary_centroid(k-1)
          - 2*left_boundary_centroid(k-3)
          + L(k-3)*bk*(-3+I)
          + (-2 + 1/2*L(k-1))*bk*(-2-2*I));
}
check_equal(vector(10,k,my(k=k-1); left_boundary_centroid_by_points(k)), \
            vector(10,k,my(k=k-1); left_boundary_centroid(k)))


left_boundary_centroid =
{
  memoize(k->
          if(k<4, return(left_boundary_centroid_by_points(k)));
          my(bk = b^(k-4));
          left_boundary_centroid(k-1)
          - 2*left_boundary_centroid(k-3)
          + 4*b*bk
          + (-3+I)*bk * L(k-3)
          - b*bk * L(k-1)
         );
}
check_equal(vector(10,k,my(k=k-1); left_boundary_centroid_by_points(k)), \
            vector(10,k,my(k=k-1); left_boundary_centroid(k)))



quit


\\-----------------------------------------------------------------------------

gL(x) = -1 + (2 + 2*x^2) / (1 - x - 2*x^3);
gR(x) = 1 - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);
C = left_boundary_centroid;

\\                                            *****
\\        g(x) =   C(0)   C(1)   C(2)   C(3)   C(4)   C(5)  ...
\\        =
\\      x*g(x) =          C(0)   C(1)   C(2)   C(3)   C(4)  ...
\\ -2*x^3*g(x) =                      -2*C(0) -2*C(1) -2*C(2)  ...
\\ gL(x*b)*x^3 =                        L(0)  L(1)*b  L(2)*b^2
\\                                          (-3+I)/b
\\ gR(x*b)*x^3 =                        R(0)  R(1)*b  R(2)*b^2
\\                                          (-2-2*I)/b
g(x) = {
         1/(1-x+2*x^3) *(
                           x^3 * (-3+I)/b * gL(x*b)
                         - x^3 * (-3+I)/b * 1
                         + x^3 * (-2-2*I)/b * gR(x*b)
                         - x^3 * (-2-2*I)/b * 1

                         - x^3 * C(2)
                         - x^2 * C(1)
                         - x * C(0)

                         + x^3 * 2*C(0)

                         + C(0)
                         + C(1)*x
                         + C(2)*x^2
                         + C(3)*x^3
                        )
         ;
}
check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))
\\ v[4]
\\ left_boundary_centroid(4)

g(x) = {
         1/(1-x+2*x^3) *(
                           x^3 * (-1 + 2*I) * (-1
                                               + (2 + 2*(x*b)^2) / (1 - (x*b) - 2*(x*b)^3))
                         + x^3 * -2 * (1
                                       - 2/(1-(x*b))
                                       + 2*(1 + (x*b) + (x*b)^2)/(1-(x*b)-2*(x*b)^3))

                         + (-1 + 4*I)*x^3
                         + (1/2 + 5/2*I)*x^2
                         + (1 + 1/2*I)*x
                         + 1/2
                        );
}
check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))

g(x) = {
         1/(1-x+2*x^3)
         *(
           + 4/b^3/(1- b*x)

           + (-1 + 2*I) * x^3 * (2 + 2*b^2*x^2) / (1 - b*x - 2*b^3*x^3)
           +  x^3 * -4*(1 + x*b + (x*b)^2)/(1 - b*x - 2*b^3*x^3)

           - 4/b*x^2 - 4/b^2*x - 4/b^3
           + (-2 + 2*I)*x^3
           + (1/2 + 5/2*I)*x^2
           + (1 + 1/2*I)*x
           + 1/2
          );
}
check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))


g(x) = {
         1/(1-x+2*x^3)
         *(
           - b/(1- b*x)

           + (
              + (-8 - 12*I)*x^5
              + (-4 - 4*I)*x^4
              + (-6 + 4*I)*x^3)
           / (1 - b*x - 2*b^3*x^3)

           + (-2 + 2*I)*x^3
           + (-3/2 + 9/2*I)*x^2
           + (1 + 5/2*I)*x
           + (3/2 + I)
          );
}
check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))

{
  my(n = (
          + (-8 - 12*I)*x^5
          + (-4 - 4*I)*x^4
          + (-6 + 4*I)*x^3  ),
     d = (1 - b*x - 2*b^3*x^3),
     r = (1/2 + 3/2*I)*x^2 + (-1/2 + 1/4*I)*x + (5/8 + 1/8*I),
     q = (1/2 - 5/2*I)*x^2 - I*x + (-5/8 - 1/8*I));
  check_equal(n/d == q + r/d, 1)
}

g(x) = {
         1/(1-x+2*x^3)
         *(
           - b/(1 - b*x)

           + ( (5/8 + 1/8*I) + (-1/2 + 1/4*I)*x + (1/2 + 3/2*I)*x^2  )
           / (1 - b*x - 2*b^3*x^3)

           + (-2 + 2*I)*x^3
           + (-1 + 2*I)*x^2
           + (1 + 3/2*I)*x
           + (7/8 + 7/8*I)
          );
}
check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))

g(x) = {
         1/2
         * 1 / (1+x)
         * 1 / ((1/2-1/2*I) - x)
         * 1 / ((1/2+1/2*I) - x)
         * 1 / (1 - b*x)
         * 1 / (1 - b*x - 2*b^3*x^3)
         *(
           - b * (1 - b*x - 2*b^3*x^3)

           + (1 - b*x)*( (5/8 + 1/8*I) + (-1/2 + 1/4*I)*x + (1/2 + 3/2*I)*x^2  )

           + (1 - b*x)*(1 - b*x - 2*b^3*x^3)
           * ( (15/8-1/8*I) + 5/2*I*x + (-1+2*I)*x^2 )
          )

         + (-1 + I)
         ;
;
}
check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))

{
  my(n = (  (16 - 32*I)*x^6
          + (8 - 16*I)*x^5
          + (-18 + 18*I)*x^4
          + (2 - 24*I)*x^3
          + (11 + 5*I)*x^2
          + (-10 + I)*x
          + (3 - 2*I)),
     d = (-32*x^7 + (16 - 16*I)*x^6 + (16 + 8*I)*x^5 - 32*x^4 + (12 - 12*I)*x^3 + (4 + 8*I)*x^2 + (-6 - 4*I)*x + 2),
     r = n%d,
     q = (n-r)/d);
  check_equal(n/d == q + r/d, 1);
  \\ print("q=",q,"\nr=",r);
}

{
  check_equal((-32*x^7 + (16 - 16*I)*x^6 + (16 + 8*I)*x^5 - 32*x^4 + (12 - 12*I)*x^3 + (4 + 8*I)*x^2 + (-6 - 4*I)*x + 2)
              ==
              4
              *((1/2-1/2*I) - x)^2
              *((1/2+1/2*I) - x)^2
              * (1+x)^2
              * (1 - b*x)
           * (1 - b*x - 2*b^3*x^3)
             )
}

\\ factor(-32*x^7 + (16 - 16*I)*x^6 + (16 + 8*I)*x^5 - 32*x^4 + (12 - 12*I)*x^3 + (4 + 8*I)*x^2 + (-6 - 4*I)*x + 2)
\\
\\ check_equal(1 - 2*x + 2*x^2 == 2*((1/2-1/2*I) - x)*((1/2+1/2*I) - x), 1)

g(x) = {
         1/2 * conj(b)/2
         * 1 / (1+x)
         * 1 / ((1/2+1/2*I) - x)
         * 1 / ((1/2-1/2*I) - x)
         * 1 / ((1/2-1/2*I) - x)
         * 1 / (1 - b*x - 2*b^3*x^3)
         *(
           - b * (1 - b*x - 2*b^3*x^3)

           + (1 - b*x)*( (5/8 + 1/8*I) + (-1/2 + 1/4*I)*x + (1/2 + 3/2*I)*x^2  )

           + (1 - b*x)*(1 - b*x - 2*b^3*x^3)
           * ( (15/8-1/8*I) + 5/2*I*x + (-1+2*I)*x^2 )
          )

         + (-1 + I);
}
check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))
\\ a=g

g(x) = {
         (1/2
          - 1/2*I * x
          + (-1/2 + 1/2*I) * x^2
          + x^3
          + (7 - 7*I)   * x^4
          + (-8 - 4*I)  * x^5
          + 8           * x^6
          + (16 - 16*I) * x^7
         )
               / (
                   (1+x)
                  * (1 - conj(b)*x)
                  * (1 -      b *x)^2
                  * (1 - b*x - 2*b^3*x^3)
                  );
}
\\ g(x)
\\ a(x)/g(x)

check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))


g(x) = {
         -1 + I
         + (2/25 - 6/25*I)           / (1           + x)
         + (-1/20 - 3/20*I)          / ((1/2+1/2*I) - x)
         + (19/50 - 42/50*I)         / ((1/2-1/2*I) - x)
         + (-1/10 + 3/10*I)          / (((1/2-1/2*I) - x)*((1/2-1/2*I) - x))
         + (1             + 2*I*x^2) / (1   + (-1-I)*x    + (4 - 4*I)*x^3);
}
check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))


g(x) = {
         -1 + I
         + 1/25*(2 - 6*I)      / (1 + x)
         - 1/10*(2 + I)        / (1 - conj(b)*x)
         + 1/50*(61 - 23*I)    / (1 - b*x)
         - 1/5*(3 + I)   / (1 - b*x)^2
         + (1  + 2*I*x^2) / (1  - b*x  - 2*b^3*x^3);
}
check_equal(Vec(g(x)-O(x^15)), \
            vector(15,k,left_boundary_centroid(k-1)))


\\-----------------------------------------------------------------------------
check_equal(Vec(1/(1+x) -O(x^15)), \
            vector(15,k,my(k=k-1); (-1)^k))
check_equal(Vec(1/(1 - b*x)^2  -O(x^15)), \
            vector(15,k,my(k=k-1); (k+1)*b^k))

dJA(k) = if (k<2,0, k==2,1, dJA(k-1) +  2*dJA(k-3))
dJA = memoize(dJA)
Vec( 1 / (1  - b*x  - 2*b^3*x^3)  -O(x^7))
vector(7,k,my(k=k-1); b^k*dJA(k+2))
check_equal(Vec( 1 / (1  - b*x  - 2*b^3*x^3)  -O(x^15)), \
            vector(15,k,my(k=k-1); b^k*dJA(k+2)))


left_boundary_centroid_by_powers(k) = \
    if(k==0, -1 + I, 0)               \
    + 1/25 * (2 - 6*I) * (-1)^k       \
    - 1/10 * (2 + I)     * conj(b)^k  \
    + 1/50 * (61 - 23*I) * b^k        \
    - 1/5  * (3 + I)     * (k+1)*b^k  \
    +      b^k     * dJA(k+2)         \
    + 2*I* b^(k-2) * dJA(k);
check_equal(vector(30,k,my(k=k-1); left_boundary_centroid(k)), \
            vector(30,k,my(k=k-1); left_boundary_centroid_by_powers(k)))

check_equal(vector(30,k,my(k=k-1); 1/2*L(k) + if(k==0,1/2,0)), \
            vector(30,k,my(k=k-1); dJA(k)+dJA(k+2)))

left_boundary_centroid_by_powers(k) = \
    if(k==0, -1 + I, 0)               \
    + 1/25 * (2 - 6*I) * (-1)^k       \
    - 1/10 * (2 + I)     * conj(b)^k  \
    + 1/50 * (31 - 33*I) * b^k        \
    - 1/5  * (3 + I)     * k*b^k      \
    + b^k * (1/2*L(k) + if(k==0,1/2,0));
check_equal(vector(30,k,my(k=k-1); left_boundary_centroid(k)), \
            vector(30,k,my(k=k-1); left_boundary_centroid_by_powers(k)))

left_boundary_centroid_by_powers(k) =           \
    if(k==0, -1/2 + I, 0)                       \
    + (-1)^k    * 1/25 * (2 - 6*I)              \
    - conj(b)^k * 1/10 * (2 + I)                \
    + b^k       * ( 1/50 * (31 - 33*I)          \
                    - 1/5  * (3 + I) * k        \
                    + 1/2*L(k)                  \
                  );
check_equal(vector(30,k,my(k=k-1); left_boundary_centroid(k)), \
            vector(30,k,my(k=k-1); left_boundary_centroid_by_powers(k)))

left_boundary_centroid_by_powers(k) = \
    if(k==0, -1/2 + I, 0)               \
    + 1/25 * (2 - 6*I) * (-1)^k       \
    - 1/10 * (2 + I)     * conj(b)^k  \
    + 1/50 * (31 - 33*I) * b^k        \
    - 1/5  * (3 + I)     * k*b^k      \
    + 1/2  * L(k)        * b^k;
check_equal(vector(30,k,my(k=k-1); left_boundary_centroid(k)), \
            vector(30,k,my(k=k-1); left_boundary_centroid_by_powers(k)))


\\-----------------------------------------------------------------------------
check_exact(gL(x*b), (1 + b*x)*(1 + 2*b^2*x^2) / (1 - b*x - 2*b^3*x^3))
check_exact(gR(x*b), (1 + b^2*x^2 + 2*b^4*x^4) / ((1-b*x) * (1 - b*x - 2*b^3*x^3)))

check_exact((-3+I)*gL(x*b) + (-2-2*I)*gR(x*b), \
            (-3+I)*(1 + b*x)*(1 + 2*b^2*x^2) / (1 - b*x - 2*b^3*x^3) \
         +  (-2-2*I)*(1 + b^2*x^2 + 2*b^4*x^4) / ((1-b*x) * (1 - b*x - 2*b^3*x^3)))

check_exact((-3+I)*gL(x*b) + (-2-2*I)*gR(x*b), \
            ( (-3+I)*(1 + b*x)*(1 + 2*b^2*x^2)*(1-b*x) \
              + (-2-2*I)*(1 + b^2*x^2 + 2*b^4*x^4)) \
            / ((1-b*x) * (1 - b*x - 2*b^3*x^3)))

check_exact((-3+I)*gL(x*b) + (-2-2*I)*gR(x*b), \
            ( (-3+I)*(1-b*x)*(1 + b*x)*(1 + 2*b^2*x^2) \
              - 2*b*(1 + b^2*x^2 + 2*b^4*x^4)) \
            / ((1-b*x) * (1 - b*x - 2*b^3*x^3)))

check_exact(( (-3+I)*(1-b*x)*(1 + b*x)*(1 + 2*b^2*x^2) \
              - 2*b*(1 + b^2*x^2 + 2*b^4*x^4)), \
            (-8 + 24*I)*x^4 + (2 - 10*I)*x^2 + (-5 - I))

check_exact(( (-3+I)*(1-b*x)*(1 + b*x)*(1 + 2*b^2*x^2) \
              - 2*b*(1 + b^2*x^2 + 2*b^4*x^4)), \
            (-8 + 24*I)*x^4 + (2 - 10*I)*x^2 + (-5 - I))

\\-------------

check_exact(gL(x*b)-1, \
            -1 + -1 + 2* (1 + b^2*x^2) / (1 - b*x - 2*b^3*x^3))
check_exact(gL(x*b)-1, \
            -2 + 2* (1 + b^2*x^2) / (1 - b*x - 2*b^3*x^3))

check_exact(gR(x*b) - 1, \
            - 2/(1-b*x) + 2*(1 + b*x + b^2*x^2)/(1-b*x-2*b^3*x^3) )

check_exact((-3+I)*(gL(x*b)-1) + (-2-2*I)*(gR(x*b)-1), \
            (-3+I)*( -2 + 2* (1 + b^2*x^2) / (1 - b*x - 2*b^3*x^3) ) \
            + (-2-2*I)*( - 2/(1-b*x) + 2*(1 + b*x + b^2*x^2)/(1-b*x-2*b^3*x^3) ))

check_exact((-3+I)*(gL(x*b)-1) + (-2-2*I)*(gR(x*b)-1), \
               (-3+I)*-2 + (-3+I)*2* (1 + b^2*x^2) / (1 - b*x - 2*b^3*x^3) \
             - (-2-2*I)*2/(1-b*x) + (-2-2*I)*2*(1 + b*x + b^2*x^2)/(1-b*x-2*b^3*x^3) )

check_exact((-3+I)*(gL(x*b)-1) + (-2-2*I)*(gR(x*b)-1), \
               (-3+I)*-2 \
             - (-2-2*I)*2/(1-b*x) \
             + (-3+I)*2* (1 + b^2*x^2)       / (1 - b*x - 2*b^3*x^3) \
             + (-2-2*I)*2*(1 + b*x + b^2*x^2)/ (1 - b*x - 2*b^3*x^3) )

\\      -3+I
\\      -3   -2   -1   0
\\           -2-I
\\           -2-2*I

check_exact((-3+I)*(gL(x*b)-1) + (-2-2*I)*(gR(x*b)-1), \
               (-3+I)*-2 \
             + (4 + 4*I)/(1-b*x) \
             + (  (-3+I)*2* (1 + b^2*x^2)   \
                 + (-2-2*I)*2*(1 + b*x + b^2*x^2)  )/ (1 - b*x - 2*b^3*x^3) )


check_exact((-3+I)*(gL(x*b)-1) + (-2-2*I)*(gR(x*b)-1), \
               (-3+I)*-2 \
             + (4 + 4*I)/(1-b*x) \
             + ( (4 - 20*I)*x^2 - 8*I*x + (-10 - 2*I)  )/ (1 - b*x - 2*b^3*x^3) )

\\-----------------------------------------------------------------------------
\\       apply(abs,polroots(1           + (-1-I)*x        + (4 - 4*I)*x^3))
\\       apply(abs,polroots(-1/2*I      + (-1+I)*x + x^2))

r = 1.695620769559862057416367100;
\\ \p 5
\\ left_boundary_centroid(k) = 2.3879^k;
\\ for(k=0,40, \
\\    print(log(abs(left_boundary_centroid(k+1))) - log(abs(left_boundary_centroid(k)))))
for(k=0,20, \
   print(right_boundary_centroid_by_points(k) / R(k) /b^k * 1.0))
\\ for(k=0,20, \
\\    print("k=",k,"  ",left_boundary_centroid(k) / L(k) /b^k * 1.0))


apply(abs,polroots(1  - b*x  - 2*b^3*x^3))
\\ 0.65108, 0.41702, 0.65108
\\ -1
\\ abs(b) = sqrt(2)
\\ abs(b^2) = sqrt(2)
\\ exp(0.87462)
\\
\\ g(x) = 1/(1+x);
\\ g(x) = 1/(1-b*x);
\\ g(x) = 1/(1-b*x)^2;    \\ = (k+1)*b^k
\\ g(x) = 1/(1-2*x)^2;    \\ = n*2^(n-1)
\\ g(x) = 1/(1-3*x)^2;    \\ = (n-1)*3^(n-2)
\\ g(x) = 1/(1-b*x-2*b^3*x^3);
\\ log(abs(Vec(g(x)-O(x^10))))
\\ Vec(g(x)-O(x^10))
\\ vector(6,k,my(k=k-1);  (k+1)*b^k)

\\ abs(vector(20,k,my(k=k-1);  (k+1)*b^k / r^k))

\\ for(i=1,50, my(k=floor(1.2^i)); print("k=",k,"  ",left_boundary_centroid(k)/b^k / L(k)*1.0))
\\ root is b*r, b*r2, b*r3



\\-----------------------------------------------------------------------------

\\ GRtotal(k) = GRtotal(k-1) + b^k*L(k-1) - I*GLN(k-1)
\\        = GRtotal(0) + sum(k=0,k-1, b^k - I*L(k-1))
\\        = GRtotal(0) + (b^k-1)/(b-1) - I* sum(k=0,k-1, L(k-1))
\\        = 1/2 + I - I*b^k - I* sum(k=0,k-1, L(k-1))

right_boundary_centroid_by_recurrence(k) = \
    if(k==0, return(1/2));               \
    right_boundary_centroid_by_recurrence(k-1) \
    + b^k * L(k-1) \
    - I*left_boundary_centroid(k-1);
check_equal(vector(8,k,my(k=k-1); right_boundary_centroid_by_recurrence(k)), \
            vector(8,k,my(k=k-1); right_boundary_centroid_by_points(k)))

gGLN(x) =                                           \
  -1/2 + I                                          \
  + 1/25 * (2 - 6*I)   / (1 + x)                    \
  - 1/10 * (2 + I)     / (1 - conj(b)*x)            \
  + 1/50 * (61 - 23*I) / (1 - b*x)                  \
  - 1/5  * (3 + I)     / (1 - b*x)^2                \
  + 1/2 * gL(b*x);

\\ g(x) = 1/2 + x*(g(x) + b*gL(b*x) - I*gGLN(x));
g(x) = 1/(1-x) * (  1/2 + x*( b*gL(b*x) - I*gGLN(x)));
check_equal(Vec(g(x)-O(x^10)), \
            vector(10,k,my(k=k-1); right_boundary_centroid_by_points(k)))

g(x) = 1/(1-x) * 1/2 + 1/(1-x) * x*b*gL(b*x) - 1/(1-x) * x*I*gGLN(x);
check_equal(Vec(g(x)-O(x^10)), \
            vector(10,k,my(k=k-1); right_boundary_centroid_by_points(k)))

g(x) = (1/2)/(1-x)  + b*x/(1-x) *gL(b*x) - x*I/(1-x) * gGLN(x);
check_equal(Vec(g(x)-O(x^10)), \
            vector(10,k,my(k=k-1); right_boundary_centroid_by_points(k)))


g(x) = { 
  (1/2)/(1-x)
  + 1 + I
  + (79 + 27*I)/(41*x - 41)
  + ((24 + 112*I)*x^2 + (24 + 112*I)*x + (52 + 24*I))/328
       /(328/328*x^3 - 82/328*I*x + (41 + 41*I)/328)
  - x*I/(1-x) * gGLN(x);
}
check_equal(Vec(g(x)-O(x^10)), \
            vector(10,k,my(k=k-1); right_boundary_centroid_by_points(k)))

g(x) = { 
  1 + I
  + (-117/82 - 27/41*I)/(1-x)
  + ((68/41 + 44/41*I)*x^2 + (68/41 + 44/41*I)*x + (38/41 - 14/41*I))
       /(1-b*x-2*b^3*x^3)
  - x*I/(1-x) * gGLN(x);
}
check_equal(Vec(g(x)-O(x^10)), \
            vector(10,k,my(k=k-1); right_boundary_centroid_by_points(k)))


g(x) = { 
    ( 57/41 - 21/41*I ) /(1 - x)
  + (3 + I)/25          /(1 + x)
  + (-1/5 - 1/10*I)     /(1 - conj(b)*x)
  + (-101/50 + 43/50*I) /(1 - b*x)
  + (3/5 + 1/5*I)       /(1 - b*x)^2

  + (8/41 - 31/41*I)
    * ( b^2*x^2 + (1 + I)*b*x + (4/5 + 3/5*I) )
    / (1-b*x-2*b^3*x^3)
  ;
}
check_equal(Vec(g(x)-O(x^10)), \
            vector(10,k,my(k=k-1); right_boundary_centroid_by_points(k)))

\\ gL(x) = -1           + 2*(1     + x^2) / (1 - x - 2*x^3);
\\ gR(x) = 1  - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);

\\ limit
GR = ( (8 - 31*I) + (39 - 23*I)*r + (25 - 20*I)*r^2 )/41 \
    / ( 1 + r + r^2 )/2;
print("GR limit ",GR)
polroots(4 - 25*x + 84*x^2 - 164*x^3) \\ imag

check_equal(GR, \
            ( (8 - 31*I) + (39 - 23*I)*r + (25 - 20*I)*r^2 )/82 \
            / (r - (-1 - I*sqrt(3))/2)/ (r - (-1 + I*sqrt(3))/2))

\\ r^3 - r^2 - 2 == 0
\\ r^3 - 1 == 1 + r^2
\\ r = 1/3 * (C + 1/C + 1)

w = sqrt(3)*I
check_equal(GR, \
            25/82 + (7 + 8*w)/(82*r + (41 - 41*w)) + (7 - 8*w)/(82*r + (41 + 41*w)) \
            + I*(-10/41 + (-9 + 19*w)/(492*r + (246 - 246*w)) + (-9 - 19*w)/(492*r + (246 + 246*w)) ))

check_equal(GR, \
            25/82 - 10/41*I \
            + (7 + 8*w)/(82*r + (41 - 41*w)) \
            + (7 - 8*w)/(82*r + (41 + 41*w)) \
            + I*(-9 + 19*w)/(492*r + (246 - 246*w)) \
            + I*(-9 - 19*w)/(492*r + (246 + 246*w)) )

check_equal(GR, \
            25/82 - 10/41*I \
            + (7 + 8*w) * conj(82*r + (41 - 41*w)) /(82*r + 41 - 41*w) / (82*r + 41 + 41*w) \
            + (7 - 8*w)/(82*r + (41 + 41*w)) \
            + I*(-9 + 19*w)/(492*r + (246 - 246*w)) \
            + I*(-9 - 19*w)/(492*r + (246 + 246*w)) )

check_equal(GR, \
            25/82 - 10/41*I \
            + (7 + 8*w) * conj(82*r + (41 - 41*w)) /(41^2*(2*r + 1)^2 + 3*41^2) \
            + (7 - 8*w)/(82*r + (41 + 41*w)) \
            + I*(-9 + 19*w)/(492*r + (246 - 246*w)) \
            + I*(-9 - 19*w)/(492*r + (246 + 246*w)) )

