\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(strictargs,1);
default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\ngot  %s\nwant %s",x,y)));
  return(0);
}

check_equal(got,want,name="") =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong ",name,"\n",d); quit(1),
     print("ok ",check_count," ",name));
}

\\-----------------------------------------------------------------------------
\\ delta Y  A137426

delta_x(k) = if(k==0,0, real(MinusPoint(2^(k-1)-1) - MinusPoint(2^(k-1))));
delta_y(k) = if(k==0,0, imag(MinusPoint(2^(k-1)-1) - MinusPoint(2^(k-1))));
vector(15,k,k--; delta_x(k))
vector(15,k,k--; delta_y(k))

delta(k) = if(k==0,0, norm(MinusPoint(2^(k-1)-1) - MinusPoint(2^(k-1))));
vector(15,k,k--; delta(k))

check_exact(vector(15,k,k--; if(k==0,0, norm(((I-1)^(k-1) -1)/(I-1 - 1) - (I-1)^(k-1)))), \
            vector(15,k,k--; delta(k)));

check_exact(vector(15,k,k--; if(k==0,0, norm((I-1)^k + 2/5-1/5*I))), \
            vector(15,k,k--; delta(k)));

check_exact(2*(-1/2+1/2*I - x)*(-1/2-1/2*I - x), (1 + 2*x + 2*x^2));
check_exact(2* (-b/2 - x) * (-conj(b/2) - x), (1 + 2*x + 2*x^2));

gdelta(x) =
{
  -2
  + 1/5 * 1/(1 - x)
  + 1/(1 - 2*x)
  + (-1/10 + 3/10*I) /(-1/2*conj(b) - x)
  + (-1/10 - 3/10*I) /(-1/2*b - x)
  ;
}
check_exact(gf_terms(gdelta(x),100), vector(100,k,k--; delta(k)));

gdelta(x) =
{
  -2
  + 1/5 * 1/(1 - x)
  + 1/(1 - 2*x)
  + (2/5 - 1/5*I)/(1 - (-b)*x)
  + (2/5 + 1/5*I)/(1 - (-conj(b))*x)
  ;
}
check_exact(gf_terms(gdelta(x),100), vector(100,k,k--; delta(k)));

delta_formula(k) =
{
  if(k==0,0,
     1/5
     + 2^k
     + (2/5 - 1/5*I)*(-b)^k
     + (2/5 + 1/5*I)*(-conj(b))^k
     );
}
check_exact(vector(100,k,k--; delta_formula(k)), vector(100,k,k--; delta(k)));

delta_formula(k) =
{
  if(k==0,0,
     2^k
     + 1/5*vector_modulo([4,-6,2,2, -4,6,-2,-2],k) * 2^floor(k/2)
     + 1/5
     );
}
vector(20,k,k--; (delta_formula(k)-delta(k))/2^floor(k/2))
check_exact(vector(100,k,k--; delta_formula(k)), \
            vector(100,k,k--; delta(k)));

{
  for(n=1,8,
     my(z = MinusPoint(n) - MinusPoint(n-1));
     print("n="n" z="z"  norm="norm(z)));
}
distance(k) = sum(n=1,2^k-1,  \
                  abs(MinusPoint(n-1) - MinusPoint(n)));
distance = memoize(distance);
check_exact(distance(0), 0);
check_exact(distance(1), 1);
check_exact(distance(2), 2+sqrt(5));
check_exact(distance(3), 4 + 2*sqrt(5) + sqrt(9));
check_exact(distance(4), 8 + 4*sqrt(5) + 2*sqrt(9) + sqrt(13));

for(k=0,10, print(distance(k)/2^k));

distance_by_recurrence(k) = if(k==0,0, 2*distance(k-1) + sqrt(delta(k)));
vector(5,k,k--; distance_by_recurrence(k))
vector(5,k,k--; distance(k))
check_equal(vector(12,k,k--; distance_by_recurrence(k)), \
            vector(12,k,k--; distance(k)));

\\ {
\\   my(k=15, total=0);
\\   for(i=0,k-1,
\\      my(f = 2^(k-1-i),
\\         s = sqrt(delta(i)),
\\         t = f * s);
\\      total += t;
\\      print(f,"  sqrt ",delta(i),"  ",t/2^k);
\\      );
\\   print(total/2^k,"     ",distance(k)/2^k);
\\ }

\\ {
\\   my(k=10, total=0);
\\   for(i=1,k,
\\      my(f = 2^(k-i),
\\         s = sqrt(delta(i)),
\\         t = sqrt((2^i
\\                  + 1/5*vector_modulo([4,-6,2,2, -4,6,-2,-2],i) * 2^floor(i/2)
\\                  + 1/5) / 2^(2*i)));
\\      total += t;
\\      print(f,"  sqrt ",delta(i),"  ",t);
\\      );
\\   print(total,"     ",distance(k)/2^k);
\\ }

{
  my(k=10, total=0);
  for(i=1,k,
     my(f = 2^(k-i),
        s = sqrt(delta(i)),

        t = sqrt((1
                 + 1/5*vector_modulo([4,-6,2,2, -4,6,-2,-2],i) / 2^ceil(i/2)
                 + 1/5 / 2^i) / 2^i));
     total += t;
     print(f,"  sqrt ",delta(i),"  ",t);
     );
  print(total,"     ",distance(k)/2^k);
}

\\ without offset 2.414213562373095048801688722 = 1+sqrt(2)

distance_scaled(k) =
{
  sum(i=1,k,
        1/sqrt(2)^i
        * sqrt(1
               + 1/5*vector_modulo([4,-6,2,2, -4,6,-2,-2],i) / 2^(ceil(i/2))
               + 1/5 * 1/2^i
              ));
}  
check_equal(vector(12,k,k--; distance_scaled(k)), \
            vector(12,k,k--; distance(k)/2^k));

my(k=5); print(distance_scaled(k),"     ",distance(k)/2^k);
print(distance_scaled(500));
print(distance_scaled(1000));
print(distance_scaled(5000));
\\print(distance_scaled(10000));

distance_scaled_by_recurrence(k) = distance_scaled(k-1) + sqrt(delta(k))/2^k;
print(distance_scaled(5));
print(distance_scaled_by_recurrence(5));

\\for(k=1,20,print(core(delta(k))))

\\ limit 2.27316887159923577765538


\\ abs(10)
\\ abs(10+(1+I))
\\ abs(10+2*(1+I))
\\ 1.04536101718726077421091384
\\ 2.16552506059643937799936849

w38 = (I-1)/sqrt(2);
check_equal(abs(w38), 1);

distance_scaled(k) =
{
  sum(i=1,k,
        1/sqrt(2)^i * abs(w38^i + (2/5-1/5*I)/sqrt(2)^i)
  );
}  
check_equal(vector(12,k,k--; distance_scaled(k)), \
            vector(12,k,k--; distance(k)/2^k));

\\     w38^1    6   w38^3
\\          \   |   /
\\            \ | /
\\ w38^4 -------o-------w38^0    8
\\            / | \
\\          /   |   \
\\        7   w38^2    5

distance_scaled(k) =
{
  sum(i=1,k,
        1/sqrt(2)^i * abs(1 + (2/5-1/5*I)/w38^i/sqrt(2)^i)
  );
}  
check_equal(vector(12,k,k--; distance_scaled(k)), \
            vector(12,k,k--; distance(k)/2^k));


\\           |
\\           |
\\           |
\\  ---------o----------
\\           |
\\           |        2-I
\\           |
distance_scaled0(k) =
{
  sum(i=1,k,
        if(i%8==0,
           1/sqrt(2)^i * abs(1 + 1/5*(2-I)/sqrt(2)^i))
  );
}

\\ sqrt((a+2)^2 + 1) = (a+2) + d
\\ d = sqrt((a+2)^2 + 1) - (a+2)
\\ d^2 = ((a+2)^2 + 1) - 2*sqrt((a+2)^2 + 1)*(a+2) + (a+2)^2
\\     = (a+2)^2 + 1 - 2*sqrt((a+2)^2 + 1)*(a+2) + (a+2)^2
