\\ Copyright 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}


Sint_is_overlap(S,n) =
{
  my(m=Map(), prev=0,z=1,dz=I);
  for(i=0,n-1,
     \\ print(prev" "z" dz="dz);
     if(mapisdefined(m,prev+z),return(1));
     mapput(m,prev+z,1);
     prev=z;
     z+=dz;
     dz *= I*(-1)^bittest(S,i));
  0;
}
Sint_is_overlap(S,n) =
{
  my(m=Map(), z=0,dz=1);
  for(i=0,n-1,
     \\ print(z" dz="dz" test "z+dz" in "m);
     z+=dz;
     if(mapisdefined(m,z),return(1));
     mapput(m,z,1);
     z+=dz;
     dz *= I*(-1)^bittest(S,i));
  0;
}
vector(10,n,n++; sum(S=0,2^(n-2)-1, Sint_is_overlap(S,n)))

Sint_endpoint(S,n) =
{
  my(m=Map(), z=0,dz=1);
  for(i=-1,n-2,
     z+=dz;
     dz *= I*(-1)^bittest(S,i));
  z;
}
\\ for(S=1,15, my(n=logint(S,2)); print(n" "S" "Sint_endpoint(S,n)));
Sbin_endpoint(S) = Sint_endpoint(S,logint(S,2));

\\ vector(20,S, norm(Sbin_endpoint(S)))
\\ not in OEIS: 1, 1, 2, 2, 2, 2, 1, 5, 1, 5, 1, 5, 1, 5, 0, 8, 4, 4, 0


Sint_is_ABC(S,n, t) =
{
  my(m=Map(), z=0,dz=1);
  for(i=-1,n-2,
     \\ print("i="i" z="z/2,"  dz="dz" test "z+dz" in "m);
     z+=dz;
     if(mapisdefined(m,z),return(0));
     mapput(m,z,1);
     z+=dz;
     dz *= I*(-1)^bittest(S,i));

  \\ print("norm of z="z" is "norm(z)" cf ",4*n);
  if(t=='A,
     if(norm(z) == 4*n, return(0));     \\ is filling
     ,
     if(norm(z) != 4*n, return(0));     \\ not filling
     );
  my(p = (1+I)*z - 1 - 2*I);
  \\ print("p="p);
  if(t=='B,
     if(!mapisdefined(m,p),return(0));   \\ would be perfect
     ,
     if(mapisdefined(m,p),
        \\ print("is not perfect");
        return(0)); 
     );

  forstep(i=n-3,-2,-1,
     \\ print("rev i="i" z=",z/2," dz="dz" test "z+dz" in "m" bit="bittest(S,i)" turn ",-I*(-1)^bittest(S,i));
     z+=dz;
     if(mapisdefined(m,z),return(0));
     mapput(m,z,1);
     z+=dz;
     dz *= -I*(-1)^bittest(S,i));

  dz = - dz;
  for(i=-1,n-2,
     \\ print("second i="i" z=",z/2," dz="dz" test "z+dz" in "m);
     z+=dz;
     if(mapisdefined(m,z),return(0));
     mapput(m,z,1);
     z+=dz;
     dz *= I*(-1)^bittest(S,i));

  forstep(i=n-3,-2,-1,
     \\ print(z" dz="dz" test "z+dz" in "m);
     z+=dz;
     if(mapisdefined(m,z),return(0));
     mapput(m,z,1);
     z+=dz;
     dz *= -I*(-1)^bittest(S,i));

  \\ print("final "z);
  z==0 || error("final z="z);
  1;
}

normend_by_Sint(n) = sum(S=0,2^(n-2)-1, norm(Sint_endpoint(S,n))==n);


\\ dx,dy = +/-1 alternately
\\ net sum x,y
\\ tx = ceil(n/2)-1 steps dx after first East, second North
\\ px +1, mx -1, net px - mx = x
\\             total px + mx = tx
\\   px = (x-tx)/2
\\   py = (y-ty)/2
\\ max px = tx   mx = 0
\\ min px = 0    mx = tx

\\ ty = floor(n/2)-1 steps dy after first East, second North

tx_of_n(n) = ceil(n/2)-1;
ty_of_n(n) = floor(n/2)-1;
check_exact(tx_of_n(3), 1);
check_exact(ty_of_n(3), 0);
check_exact(tx_of_n(4), 1);
check_exact(ty_of_n(4), 1);

\\ Sum_{x,y with x^2+y^2=n and x==c mod 2 and y==f mod 2}
\\   binomial(c-1,abs((x-c)/2)) * binomial(f-1,abs((y-f)/2))
\\ where c=ceiling(n/2), f=floor(n/2)

normend_by_binomials(n) =
{
\\   my(tx = tx_of_n(n),      \\ ceil(n/2)-1;
\\      ty = ty_of_n(n),      \\ floor(n/2)-1;
\\      c=ceil(n/2),
\\      f=floor(n/2));
\\   \\ print("tx="tx" ty="ty);
\\   my(ret=0);
\\   \\ forstep(x=2-ceil(n/2),ceil(n/2),2,
\\   \\    forstep(y=2-floor(n/2),floor(n/2),2,
\\ 
\\   my(s=sqrtint(n));
\\   sum(x=-s,s,
\\       if(x%2 == c%2,
\\          binomial(c-1,abs((x-c)/2))
\\          * sum(y=-s,s,
\\                if(y%2 == f%2 && x^2 + y^2 == n,
\\ 
\\         my(px=abs((x-1-tx)/2),
\\            py=abs((y-1-ty)/2));
\\         px==floor(px) || error();
\\         py==floor(py) || error();
\\         
\\         \\ print("  x="x" px="px);
\\         \\ print("   x binom "binomial(tx,px));
\\         \\ print("  y="y" py="py);
\\         \\ print("   y binom "binomial(ty,py));
\\ 
\\         tx == ceil(n/2)-1  || error();
\\         ty == floor(n/2)-1  || error();
\\         px == abs((x-ceil(n/2))/2)  || error();
\\         py == abs((y-floor(n/2))/2)  || error();
\\         \\ ret += binomial(tx,px) * binomial(ty,py);
\\ 
\\                   binomial(f-1,abs((y-f)/2));
\\      ))));
\\ \\  ret;

  \\ c steps of x is abs(x)<=c
  \\ f steps of y is abs(y)<=f
  \\ x^2 + y^2 <= c^2 + f^2
  \\           <= ((n+1)/2)^2 + (n/2)^2
  \\            = n^2/2 + 1/2*n + 1/4
  \\ x^2 <= n so limit x<=sqrtint(n)

  \\ my(c=ceil(n/2), f=floor(n/2), s=sqrtint(n)); 
  \\ sum(x=-s,s, if((x-c)%2==0, binomial(c-1,abs((x-c)/2))
  \\   * sum(y=-s,s, if((y-f)%2==0 && x^2+y^2==n, binomial(f-1,abs((y-f)/2))))));

  \\ y-f even
  \\ -y+f even

  my(c=ceil(n/2), f=floor(n/2), s=sqrtint(n), y); 
  sum(x=-s,s,
      if((x-c)%2==0 && issquare(n-sqr(x),&y),
         (y-f)%2==0 || error();
         x <= c || error();
         y <= f || error();
         binomial(c-1,(c-x)/2)
         * (binomial(f-1,(f-y)/2)
            + if(y, binomial(f-1,(f+y)/2)))));
}


        


\\ A004018 theta square grid
norm_num_solutions(n) = \
  my(s=sqrtint(n)); sum(x=-s,s, sum(y=-s,s, x^2 + y^2 == n));
\\ vector(20,n,norm_num_solutions(n))

vector(50,n,normend_by_binomials(n))

\\ factor(2762261552050)
print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n");
for(n=0,49,print1(normend_by_binomials(n)",");if(n==24||n==37,print()));


normend_by_binomials(2)
{
  for(n=2,17,
     print("n="n" "normend_by_Sint(n)" "normend_by_binomials(n)));
}

my(z=1+I); check_exact(z+I*z-1-I + z+I*z-1-I+1,  -1 + 2*I);
\\ Sint_is_ABC(3,4, 'C)
Sint_is_ABC(6,5, 'C)
for(n=2,30, print1(sum(S=0,2^(n-2)-1, Sint_is_ABC(S,n,'C)), ","));
quit


\\-----------------------------------------------------------------------------


{
  \\ if n odd  then x,y opposite parity
  \\   (2X+1)^2 = 4X^2 + 4X + 1 is 1 mod 4
  \\   (2Y)^2   = 4Y^2 mod 4
  \\   only n = 1 mod 4 = 4N+1
  \\   c = ceil(n/2) = 2N+1 odd   so must x odd
  \\   f = floor(n/2) = 2N even   so must y even
  \\   so y-f and -y-f both even
  \\
  \\ if n = 0 mod 4 then x,y both even
  \\   c,f both = n/2 even
  \\   
  \\ if n = 2 mod 4 then x,y both odd,
  \\   c,f both = n/2 odd
  \\   

  for(n=1,-100,
     n%2==0 || next;
     my(c=ceil(n/2), f=floor(n/2), s=sqrtint(n), y); 
     for(x=-s,s,
        if((x-c)%2==0 && issquare(n-sqr(x),&y),
           print("n="n" x="x" y="y"   c="c" f="f"  ",(y-f)%2,"  ",(-y-f)%2))));
  \\ quit
}

\\-----------------------------------------------------------------------------
\\
\\ for(n=1,20, for(x=0,sqrtint(n), if(issquare(n-sqr(x),&y), print1(x","))))
\\ A283305 list x coordinates
\\ A283306 list y coordinates