#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 13;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$n_lo = BlobN($level)-3;
$n_hi = BlobN($level+1);

$Printwrap = 2;
printwrap("\\node at (0,0) [my dot,name=start];\n");
printwrap("\\node at ($x_hi,$y_hi) [my dot,name=end];\n");

my $segments = MyLineSegments->new;
my $druns    = MyLineSegments->new;
my $prev_Dpred = 0;
my $Dpred_n = 0;
my $max = 0;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ <= $n_hi} @n_list;
  my $Dpred = (@n_list == 2);

  $segments->add($x,$y, $path->n_to_xy($n+1));

  if ($Dpred && ! $prev_Dpred) {
    # start of run
    $Dpred_n = $n;
  } elsif ($prev_Dpred && ! $Dpred) {
    # end of run
    my $len = $n - $Dpred_n - 1;
    if ($len > $max) {
      $max = $len;
      foreach my $n ($Dpred_n .. $n-2) {
        $druns->add($path->n_to_xy($n), $path->n_to_xy($n+1));
        $segments->remove($path->n_to_xy($n), $path->n_to_xy($n+1));
      }
      print "% run $Dpred_n to ",$n-1,"\n";
    }
  }
  $prev_Dpred = $Dpred;
}

$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw[ultra thick]");
$Printwrap += 2;
$druns->printwrap;
printwrap(";\n");

print "  % end generated\n";
exit 0;
