#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::Geometry::Planar;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyDragon 'BlobN','transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 11;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my $n_start = BlobN($level);
my $n_end = BlobN($level+1)-3;

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

{
  my ($x,$y) = $path->n_to_xy($n_start);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_end);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (end) at ($x,$y);\n";
}

my $segments = MyLineSegments->new;
my $points = [];
foreach my $n ($n_start .. $n_end-1) {
  my ($x,$y) = $path->n_to_xy($n);
  $segments->add($x,$y, $path->n_to_xy($n + 1));
  push @$points, [$x,$y];
}
push @$points, [$path->n_to_xy($n_end)];

$Printwrap = 2;
printwrap("\\draw[my dragon line] ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");

my $planar = Math::Geometry::Planar->new;
$planar->points($points);
if (@$points >= 5) {
  $planar = $planar->convexhull2;
  $points = $planar->points;
}

$Printwrap = 2;
printwrap("\\draw[my hull line]\n");
$Printwrap = 4;
my $join = '';
foreach my $p (@$points) {
  my ($x,$y) = @$p;
  ($x,$y) = transform($x,$y);
  printwrap("$join($x,$y)");
  $join = '--';
}
printwrap("${join}cycle;\n");

$Printwrap = 2;
foreach my $i (0 .. $#$points) {
  my $p = $points->[$i];
  my ($x,$y) = @$p;
  ($x,$y) = transform($x,$y);
  my $P = $i+1;
  printwrap(" \\node at ($x,$y) [my dot,name=P$P];\n");
}
printwrap_newline();

# foreach my $i (0 .. $#$points) {
#   my $p = $points->[$i];
#   my ($x,$y) = @$p;
#   my $P = $i+1;
#   print "% GP-Test   P$P($level) == ($x)+I*$y\n";
# }

print "  % end generated\n";
exit 0;
