\\ Copyright 2015, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

default(recover,0);
check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

vector_mean(v) = vecsum(v)/#v;


\\-----------------------------------------------------------------------------
\\ max width new highs

\\ at depths
\\ [0, 3, 8, 16, 28, 66, 77, 111, 189, 320, 383, 544, 921, 1563, 2650]
\\ widths
\\ [1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, 18, 23]
\\    1  1  1  1  1  1  1  2   2   1   2   3   5


{
  my(hi=0);
  for(d=0,10000,
     my(w = WidthS_infinity(d));
     if(w > hi,
        hi = w;
        \\ print("d="d"  width="w);
        print(w);
       );
  );
}
quit


\\-----------------------------------------------------------------------------
\\ widths which occur

{
  my(seen = vector(100,i,-1), upto=1);
  for(d=0,10000,
     my(w = WidthS_infinity(d));
     if(w <= #seen && seen[w] < 0,
        seen[w] = d;
        while(upto <= #seen && seen[upto] >= 0,
              print("d="upto"  "seen[upto]);
              upto++));
  );
}
quit

\\-----------------------------------------------------------------------------
\\ mean width 2^k+2^(k-3)+... / DOne(m) unbounded

{
  my(total=0, count=0);
  for(d=0,10000,
     total += WidthS_infinity(d);
     count++;
     printf("d=%d  %.6f\n", d, total/count);
  );
}
quit


\\-----------------------------------------------------------------------------

WidthS_formula(k,d) = WidthS(k-1,d) + WidthC(k-1,d-RQ(k-1));
WidthC_formula(k,d) = WidthS(k-1,d) + WidthC(k-1,d-LQ(k-1));
WidthS(k,d) =
{
  if(k<0,error());
  if(d<0,0, k==0,d==0, WidthS_formula(k,d));
}
WidthC(k,d) =
{
  if(k<0,error());
  if(d<0,0, k==0,d==0, WidthC_formula(k,d));
}

WidthS_infinity(d) = \
  my(k=1); while(d >= RQ(k-1), k++); WidthS(k,d);
WidthC_infinity(d) = \
  my(k=1); while(d >= LQ(k-1), k++); WidthC(k+5,d);



\\ WidthS(k,d) = WidthS(k-1,d) + WidthC(k-1, d-RQ(k-1))
\\
\\ WidthS(k,d) =                 WidthC(k-1, d-RQ(k-1))
\\               WidthS(k-2,d) + WidthC(k-2, d-RQ(k-2))
\\
\\ WidthS(k,d) =                 WidthC(k-1, d-RQ(k-1))
\\                             + WidthC(k-2, d-RQ(k-2))
\\           ... + WidthS(0,d) + WidthC(0, d-RQ(0))
\\
WidthS_by_sum(k,d) =
{
  \\ WidthS(0,d) + sum(j=0,k-1, WidthC(j,d-RQ(j)));
  sum(j=0,k-1, WidthC(j,d-RQ(j))) + (d==0);
}
check_exact(matrix(10,20,k,d,k--;d--; WidthS_by_sum(k,d)), \
            matrix(10,20,k,d,k--;d--; WidthS(k,d)))


k_where_WidthC_non_zero(d) =
{
  if(d==0,return(0));
  forstep(k=50,0,-1,
         if(WidthC(k-1,d-RQ(k-1)) > 0,
            WidthC(k-1+1,d-RQ(k-1+1)) == 0 || error("oops, above");
            return(k)));
  error();

  my(k=1);
  while(TA_ecc(k-1) < d-RQ(k-1), k++);
  \\ print("to ",k);
  while(WidthC(k-1,d-RQ(k-1)) > 0, k++);
  k-1;
}
check_exact(vector(50,d,d+=5; k_where_WidthC_non_zero(d)), \
            vector(50,d,d+=5; WidthS_infinity_k(d)), \
            "k_where_WidthC_non_zero()");

k_where_WidthC_non_zero_by_ecc(d) =
{
  forstep(k=50,0,-1,
         if(d-RQ(k-1) >= 0
            \\ &&  d-RQ(k-1) <= TA_ecc(k-1)
            ,
            \\ print("found k="k" dr="d-RQ(k-1));
            WidthC(k-1+1,d-RQ(k-1+1)) == 0 || error("oops, above");
            WidthC(k-1,d-RQ(k-1)) > 0 || error("oops, this");
            return(k)));
  error();
}
check_exact(vector(50,d,d+=5; k_where_WidthC_non_zero_by_ecc(d)), \
            vector(50,d,d+=5; k_where_WidthC_non_zero(d)), \
            "k_where_WidthC_non_zero_by_ecc()");

\\ maximum k where 0 <= d-RQ(k-1) <= TA_ecc(k-1)
\\                 RQ(k-1) <= d <= RQ(k-1) + RQ(k-1)-1
\\                 RQ(k-1) <= d < 2*RQ(k-1)
\\ these ranges overlap so demand at the low end
\\                 d-RQ(k-2) > TA_ecc(k-2)
\\                 d >= 2*RQ(k-2)
\\ which is
\\     2*RQ(k-2) <= d < 2*RQ(k-1)
\\
k_where_WidthC_non_zero_by_range(d) =
{
  for(k=1,100,
     if(d-RQ(k-1) >= 0 && d-RQ(k) < 0,
        WidthC(k-1,d-RQ(k-1)) > 0 || error("oops, this");
        WidthC(k-1+1,d-RQ(k-1+1)) == 0 || error("oops, above");
        return(k)));
  error();
}
check_exact(vector(50,d,d+=5; k_where_WidthC_non_zero_by_range(d)), \
            vector(50,d,d+=5; k_where_WidthC_non_zero(d)), \
            "k_where_WidthC_non_zero_by_range()");

\\ {
\\   \\ eccentricity vs width
\\   my(k=6);
\\   print(TA_ecc(k));
\\   for(d=0,50,print(d" "WidthC(k,d)));
\\   quit();
\\ }

\\ {
\\   print("ranges of WidthC eccentricity overlap a bit");
\\   \\ d-RQ(k-1) >= 0                d >= RQ(k-1)
\\   \\ d-RQ(k-1) <= TA_ecc(k-1)      d <= RQ(k-1) + TA_ecc(k-1
\\   for(k=1,10,print(k " " RQ(k-1) " " RQ(k-1)+TA_ecc(k-1)));
\\   quit();
\\ }

\\ top range RQ(k) > d >= RQ(k-1)
\\ WidthS(k-1) = 0      WidthC(k-1,dr) = 1
\\ WidthS(k-2,dr) = 1   WidthC(k-2,dr - LQ(k-2)) = 0
\\ WidthS(k-3,dr) = 1   WidthC(k-3,dr - LQ(k-3)) = 0
\\ WidthS(k-4,dr) = 1
{
  \\ k=8 range       RQ(8)=73 > RQ(7)=43
  for(d=208,355,  \\ k=11 262,263
     my(k = WidthS_infinity_k(d));
     my(dr = d-RQ(k-1));
     my(s1 = WidthS(k-1,d), c1 = WidthC(k-1,dr));

     \\ top k is RQ(k) > d >= RQ(k-1)

     print1(d" k="k"  "s1" "c1);
     if (d <= TAdiameter(k-1),
         if(TAdiameter(k-1)!=LQ(k)-1,error());
         print(" xx  d<=diam(k-1)="TAdiameter(k-1)" so s1!=0  cf LQ(k)="LQ(k));
         next());

     \\ demanding WidthS(k-1,d)=0 is d > TAdiameter(k-1) = LQ(k)-1
     \\ so d >= LQ(k) which is bigger than d >= RQ(k-1), and gives
     \\ d-RQ(k-1) >= LQ(k)-RQ(k-1) == dJA(k-1)
     vector(50,k, LQ(k)-RQ(k-1)) == vector(50,k, dJA(k-1)) || error();
     \\
     \\ top RQ(k)-RQ(k-1) = LQ(k-1)
     vector(50,k, RQ(k)-RQ(k-1)) == vector(50,k, LQ(k-1)) || error();
     \\
     \\ so now  LQ(k-1) > d-RQ(k-1) >= dJA(k-1)

     \\---------
     \\ k-2

     my(drl = dr - LQ(k-2));
     my(s2 = WidthS(k-2,dr), c2 = WidthC(k-1,drl));
     print1(" dr="dr" diam(k-2)="TAdiameter(k-2)" drl="drl" "s2" "c2);

     \\ TAdiameter(k-2) = LQ(k-1)-1
     vector(50,k, TAdiameter(k-2)) == vector(50,k, LQ(k-1)-1) || error();
     \\ so TAdiameter(k-2) >= dr and so WidthS(k-2,dr) > 0
     TAdiameter(k-2) >= dr || error();

     \\ ecc(k-2) = RQ(k-2)-1  >= drl ?
     vector(50,k,k+=3; TA_ecc(k-2)) == vector(50,k,k+=3; RQ(k-2)-1) || error();
     TA_ecc(k-2) >= drl || error();
     \\
     \\ drl maximum LQ(k-1)-1 - LQ(k-2)
     vector(50,k,k+=3; RQ(k-2)-1) == vector(50,k,k+=3; (LQ(k-1)-1-LQ(k-2)) + dJA(k)) || error();
     vector(50,k,k+=3; (LQ(k-1)-1-LQ(k-2))) == vector(50,k,k+=3; RQ(k-2)-1 - dJA(k)) || error();
     vector(50,k,k+=3; TA_ecc(k-2) - (LQ(k-1)-1-LQ(k-2))) == vector(50,k,k+=3; dJA(k)) || error();
     \\ so TA_ecc(k-2) >= drl, for k>=2

     \\ so to have WidthC(k-2,drl) = 0 want drl < 0
     \\ dr - LQ(k-2) < 0
     \\ dr < LQ(k-2)

     if(drl >= 0,
        print("   xx as drl>=0 WidthC>0");
        next());

     \\---------
     \\ k-3
     \\ now  LQ(k-2) > dr >= dJA(k-1)

     my(drr3 = dr - RQ(k-3));
     my(s3 = WidthS(k-3,dr), c3 = WidthC(k-3,drr3));
     print1(" diam(k-3)="TAdiameter(k-3)" drr3="drr3" "s3" "c3);

     \\ TAdiameter(k-3) = LQ(k-2)-1
     vector(50,k,k+=2; TAdiameter(k-3)) == vector(50,k,k+=2; LQ(k-2)-1) || error();
     \\ so TAdiameter(k-3) >= dr and so WidthS(k-3,dr) > 0   again
     TAdiameter(k-3) >= dr || error();

     \\ ecc(k-3) = RQ(k-3)-1  >= drr3 = dr - RQ(k-3) ?
     vector(50,k,k+=3; TA_ecc(k-3)) == vector(50,k,k+=3; RQ(k-3)-1) || error();
     TA_ecc(k-3) >= drr3 || error();
     \\
     \\ drr3 maximum LQ(k-2)-1 - RQ(k-3)
     vector(50,k,k+=4; 2*LQ(k-4)) == vector(50,k,k+=4; RQ(k-3)-1 - (LQ(k-2)-1 - RQ(k-3))) || error();
     \\ so TA_ecc(k-3) >= drr, for k>=4

     \\ so to have WidthC(k-3,drr3) = 0 want drr3 < 0
     \\ dr - RQ(k-3) < 0
     \\ dr < RQ(k-3)  is smaller than LQ(k-2)

     if(drr3 >= 0,
        print("   xx as drr3>=0 WidthC>0");
        next());


     \\---------
     \\ k-4
     \\ now  RQ(k-3) > dr >= dJA(k-1)

     \\ cf top range RQ(k) > d >= RQ(k-1)
     \\ dJA(k-1) >= RQ(k-4) ?
     \\ vector(50,k,k+=4; dJA(k-1) >= RQ(k-4))

     my(drr4 = dr - RQ(k-4));
     my(s4 = WidthS(k-4,dr), c4 = WidthC(k-4,drr4));
     print1(" diam(k-4)="TAdiameter(k-4)" drr4="drr4" "s4" "c4);


     \\---------
     \\ k-5
     my(drr5 = dr - RQ(k-5));
     my(s5 = WidthS(k-5,dr), c5 = WidthC(k-5,drr5));
     print1(" drr5="drr5" "s5" "c5);

     \\---------
     \\ k-6
     my(drrl = drr5 - LQ(k-6));
     my(s6 = WidthS(k-6,drr5), c6 = WidthC(k-6,drrl));
     print1(" "s6" "c6);


     print("");

  );
}


\\ RQ(0)
\\ RQ(1)
\\ RQ(2)