\\ Copyright 2016, 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);
read("fracper.gp");

check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
\\ fracper_apply()

check_exact(fracper_apply(d->9-d, [0,0,1, [2,3]]), [9,9,8, [7,6]]);
check_exact(fracper_apply(d->9-d, [9,9,8, []]), [0,0,1, []]);
check_exact(fracper_apply(d->9-d, [ [2,3]]), [ [7,6]]);


\\-----------------------------------------------------------------------------
\\ fracper_INTERNAL_vector_periodic_reduce()

check_exact(fracper_INTERNAL_vector_periodic_reduce([]), []);
check_exact(fracper_INTERNAL_vector_periodic_reduce([0,0,0]), []);
check_exact(fracper_INTERNAL_vector_periodic_reduce([1,2,1,2]), [1,2]);

\\-----------------------------------------------------------------------------
\\ fracper_vector_rotate_up()
\\ fracper_vector_rotate_down()

check_exact(fracper_vector_rotate_up([]), [], "up []");
check_exact(fracper_vector_rotate_up([1]), [1], "up [1]");
check_exact(fracper_vector_rotate_up([10,20]), [20,10], "up [10,20]");
check_exact(fracper_vector_rotate_up([10,20,30]), [30,10,20], "up [10,20,30]");
check_exact(fracper_vector_rotate_up([10,20,30],1), [30,10,20]);
check_exact(fracper_vector_rotate_up([10,20,30],2), [20,30,10]);
check_exact(fracper_vector_rotate_up([10,20,30],3), [10,20,30]);

check_exact(fracper_vector_rotate_down([]), []);
check_exact(fracper_vector_rotate_down([1]), [1]);
check_exact(fracper_vector_rotate_down([10,20]), [20,10]);
check_exact(fracper_vector_rotate_down([10,20,30]), [20,30,10]);

\\-----------------------------------------------------------------------------
\\ fracper_normalize()

check_exact(45/99, 5/11);
check_exact( 1/10*45/99, 1/22);

check_exact(fracper_normalize([0, [1,0]]), [ [0,1]], "normalize 0, 1,0");
check_exact(fracper_normalize([0, [0,1,0]]), [ [0,0,1]], "normalize");
check_exact(fracper_normalize([1,0,0, [1,1]]), [1,0,0, [1]]);
check_exact(fracper_normalize([1,0,0, [0]]), [1, []]);
check_exact(fracper_normalize([0,0,0, [0]]), [ []]);


\\-----------------------------------------------------------------------------
\\ fracper_lex()

check_exact(cmp(simplify([ 1+I-I, [2+I-I]]), [1,[2]]), 0, "cmp");
check_exact(cmp(         [ 1+I-I, [2+I-I]] , [1,[2]]), 0, "cmp");

check_exact(fracper_lex([0,[1,0]], [0,[1,0]]), 0, "lex equal");
check_exact(fracper_lex([1, []], [2, []]), -1);
check_exact(fracper_lex([1,0,1,0,1,0,1,0, [1,2]], [ [1,0]]), 1, "lex bigger");
check_exact(fracper_lex([ [1,0,1,0,1,2]], [ [1,0]]), 1, "lex bigger");
check_exact(fracper_lex([0,0,0,0,[]], [[]]), 0);
check_exact(fracper_lex([0,0,0,0,[]], [[1]]), -1, "smaller");
check_exact(fracper_lex([0,0,0,0,[1]], [[]]), 1);

\\-----------------------------------------------------------------------------
\\ fracper_add()

\\ check_exact(fracper_add([1,2,3,[4,5]],[100,200,300,[400,500]]), \
\\             [101,202,303,[404,505]]);
\\ check_exact(fracper_add([1,2,3,[]],[100,200,[]]), \
\\             [101,202,3,[]]);
\\ 
\\ check_exact(frac_to_fracper(1/7), [[0, 0, 1]]);
\\ check_exact(frac_to_fracper(1/13),[[0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1]]);
\\ check_exact(fracper_add(frac_to_fracper(1/7),frac_to_fracper(1/13)), \
\\             frac_to_fracper(1/7+1/13));
\\ quit

\\-----------------------------------------------------------------------------
\\ fracper_to_frac()

check_exact(Pol([0,1,0]),x);
check_exact(Pol([1,0]),x);
check_exact(Pol([0,1]),1);

check_exact(fracper_to_frac([ [1, 4, 2, 8, 5, 7]]), 1/7, "1/7 decimal");
check_exact(fracper_to_frac([ [0,9]]), 1/11, "1/11 decimal");
check_exact(fracper_to_frac([0, [4,5]]), 1/22, "1/11 decimal");
check_exact(fracper_to_frac([ [0,0,1]],2), 1/7, "0,0,1 repeat");
check_exact(fracper_to_frac([ [0,1]],2), 1/3, "0,1 repeat");
check_exact(fracper_to_frac([ [1,0]],2), 2/3, "1,0 repeat");
check_exact(fracper_to_frac([0, [1,0]],2), 1/3, "1,0 repeat, half");

check_exact(fracper_to_frac([ [9]]), 1, "9 repeat");
check_exact(fracper_to_frac([ [9,9]]), 1, "9 repeat, two");
check_exact(fracper_to_frac([ [9,9,9]]), 1, "9 repeat, three");
check_exact(fracper_to_frac([ [1]],2), 1, "1 repeat");
check_exact(fracper_to_frac([ [1,1]],2), 1, "1 repeat, two");
check_exact(fracper_to_frac([ [1,1,1]],2), 1, "1 repeat, three");
check_exact(fracper_to_frac([1, []],2), 1/2, "1 only = 1/2");
check_exact(fracper_to_frac([0, [1]],2), 1/2, "1 repeat = 1/2");
check_exact(fracper_to_frac([0, [1,1]],2), 1/2, "1 repeat two, half");
check_exact(fracper_to_frac([1, [1,1]],2), 1, "[1], [1,1]");
check_exact(fracper_to_frac([ [0]],2), 0, "[], [0]");
check_exact(fracper_to_frac([0, [0]],2), 0, "[0], [0]");

test_poly_terms(p,n) =
{
  my(v=vector(n));
  for(i=1,#v,
     p*=variable(p);
     v[i]=numerator(p)\denominator(p);
     p-=v[i]);
  v;
}
check_exact(test_poly_terms(1/x + 2/x^2, 4), [1,2,0,0]);
check_exact(test_poly_terms(1/x + 2/x^3, 4), [1,0,2,0]);

{
  my(want_poly = 1/x + 1/x*3/(x^2-1));
  check_exact(test_poly_terms(want_poly,7), [1,0,3,0,3,0,3], "poly terms");
  check_exact(fracper_to_frac([1,[0,3]],'x), want_poly, "addhelp example");
}


\\-----------------------------------------------------------------------------
\\ frac_to_fracper()

check_exact(frac_to_fracper(1/8, 4), [0,2, []], "1/8 base 4");

check_exact(frac_to_fracper(1/11,10), [ [0,9]], "1/11");
check_exact(frac_to_fracper(1/22,10), [0, [4,5]], "1/22");
check_exact(frac_to_fracper(1/7,10), [ [1, 4, 2, 8, 5, 7]]);
check_exact(frac_to_fracper(1/2,2), [1, []]);
check_exact(frac_to_fracper(1/4,2), [0,1, []]);
check_exact(frac_to_fracper(1/4,2), [0,1, []]);
check_exact(frac_to_fracper(1/3,2), [ [0,1]], "frac_to_fracper() 1/3");
check_exact(frac_to_fracper(2/3,2), [ [1,0]], "2/3");
check_exact(frac_to_fracper(1/7,2), [ [0,0,1]], "1/7");
check_exact(frac_to_fracper(3/7,2), [ [0,1,1]], "3/7");
check_exact(frac_to_fracper(1/14,2), [0, [0,0,1]], "1/14");
check_exact(frac_to_fracper(16/17,2), [ [1, 1, 1, 1, 0, 0, 0, 0]], "8/9");
check_exact(frac_to_fracper(1023/1024,2), [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, []]);
check_exact(frac_to_fracper(1,2), [ [1]], "1/1 binary");
check_exact(frac_to_fracper(1/13,2), [ [0,0,0,1, 0,0,1, 1,1,0, 1,1]]);

{
  for(base=2,12,
     print("base=",base);
     for(den=1,30,
        print("den=",den);
        for(num=0,den,
           my(f=num/den);
           \\ print("try f="f);
           my(v=frac_to_fracper(f,base));
           \\ print("  v="v);
           my(got=fracper_to_frac(v,base));
           check_exact(got,f))));
}


\\-----------------------------------------------------------------------------
\\ fracper_unroll()

check_exact(fracper_unroll([ []]), [0, []]);
check_exact(fracper_unroll([ [1]]), [1, [1]]);
check_exact(fracper_unroll([ [1,2,3]]), [1, [2,3,1]]);
check_exact(fracper_unroll([1, []]), [1,0, []]);
check_exact(fracper_unroll([1, [9]]), [1,9, [9]]);
check_exact(fracper_unroll([1,2, [7,8,9]]), [1,2,7, [8,9,7]]);


\\-----------------------------------------------------------------------------

{
  system("./my-gp-symbols.pl --exclude-prefix fracper_ --exclude-prefix frac_ fracper.gp >fracper-test-symbols.tmp");
  my(test_symbols=readstr("fracper-test-symbols.tmp"));
  system("rm fracper-test-symbols.tmp");
  print(#test_symbols," symbols");
  check_exact(#test_symbols>0,1);
  for(test_i=1,#test_symbols,
     my(test_symbol=test_symbols[test_i]);
     check_exact(eval(test_symbol), eval(Str("'"test_symbol)), test_symbol));
}

\\-----------------------------------------------------------------------------
