#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon '$triple_5ths_and_next_within';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

my $max_level = 10;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script\n";
printwrap_indent("% GP-DEFINE  ");
printwrap("gen_fifths_difference_values = {[\n");
$Printwrap += 2;

my @confs = (MyDragonConf->initial (want_next_segs => 3,
                                    within => $triple_5ths_and_next_within));
{
  my $bad = 0;
  foreach my $k (0 .. $max_level) {
    ### $k
    my $join = '';
    my @diffs;
    foreach my $conf (@confs) {
      ### diffs: [$conf->triple_5ths_diffs]
      ### to next: [$conf->triple_5ths_diff_to_next]
      push @diffs, $conf->triple_5ths_diffs, $conf->triple_5ths_diff_to_next;
    }
    printwrap("[ ");
    $Printwrap += 2;
    foreach my $i (0 .. $#diffs) {
      printwrap($diffs[$i] . ($i==$#diffs ? '' : ','));
    }
    $Printwrap -= 2;
    printwrap(" ]" . ($k==$max_level ? '' : ',') . "\n");
    @confs = map {$_->expand(0), $_->expand(1)} @confs;
  }
}
printwrap("]};\n");

print "% end generated\n";
exit 0;
