#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90','rect_corners_sort',
    'BlobN','Lpred_k','Rpred';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 7;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my $n_lo = BlobN($level)        - 1;
my $n_hi = BlobN($level+1) - 3  + 1;

print "  \\newcommand{\\MyLevel}{$level}\n";

$Printwrap = 2;
{
  my ($x,$y) = $path->n_to_xy($n_lo);
  printwrap("\\coordinate (start) at ($x,$y);\n");
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  printwrap("\\coordinate (end) at ($x,$y);\n");
}
{
  my ($x,$y) = $path->n_to_xy($n_lo + 1);
  printwrap("\\node at ($x,$y) [my dot];\n");
}
{
  my ($x,$y) = $path->n_to_xy($n_hi - 1);
  printwrap("\\node at ($x,$y) [my dot];\n");
}

my $BlobL = -2;   # -2 adjust for running BlobN(k)-1 to BlobN(k+1)+1
my $BlobR = -2;
my %seen;  # boundary squares
my %squares;

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  $segments->add($x,$y, $x2,$y2);

  if (Lpred_k($level,$n)) {
    $BlobL++;
    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
    $x2 += $lx;
    $y2 += $ly;
    ($x,$y, $x2,$y2) = rect_corners_sort ($x,$y, $x2,$y2);
    if (! $seen{"$x,$y"}++) {
      push @{$squares{'L'}}, "$x,$y";
    }
  }
  if (Rpred($n)) {
    $BlobR++;
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
    $x2 += $rx;
    $y2 += $ry;
    ($x,$y, $x2,$y2) = rect_corners_sort ($x,$y, $x2,$y2);
    if (! $seen{"$x,$y"}++) {
      push @{$squares{'R'}}, "$x,$y";
    }
  }
}

{
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

foreach my $side ('L','R') {
  my $aref = $squares{$side};
  $Printwrap = 2;
  printwrap("\\fill[my ${side}Q colour] foreach \\p in {");
  $Printwrap += 2;
  foreach my $xy (@$aref) {
    printwrap("($xy),");
  }
  printwrap("}\n");
  printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");
}

my $BlobLQ = scalar(@{$squares{'L'}});
my $BlobRQ = scalar(@{$squares{'R'}});
my $BlobBQ = $BlobLQ + $BlobRQ;
print "  \\newcommand{\\MyBlobLQ}{$BlobLQ}\n";
print "  \\newcommand{\\MyBlobRQ}{$BlobRQ}\n";
print "  \\newcommand{\\MyBlobBQ}{$BlobBQ}\n";
foreach my $side ('L','R') {
  print "% GP-Test-Last  Blob${side}Q($level) == ",
    scalar(@{$squares{$side}}),"\n";
}
print "% GP-Test  BlobL($level) == $BlobL\n";
print "% GP-Test  BlobR($level) == $BlobR\n";

print "  % end generated\n";
exit 0;
