#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','transform_to_horizontal','JA';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
# prefer $level even so segments are horizontal
my $level = 16;

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_mid) = $path->level_to_n_range($level);
my $n_hi = 2*$n_mid;
my $scale = 2**($level/2);
print "  % $n_lo to $n_mid to $n_hi\n";
print "  \\begin{scope}[scale=1/$scale]\n";

$Printwrap = 2;
printwrap("\\fill foreach \\p in {");

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

$Printwrap = 4;
my %seen;
my $count = 0;
N: foreach my $n ($n_lo .. $n_mid-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);

  my $mx = $x;
  my $my = $y;
  my $above = 0;
  my $below = 1;
  foreach (1 .. 3) {
    $x += $dx;
    $y += $dy;
    ($dx,$dy) = xy_rotate_plus90($dx,$dy);

    if ($x <= $mx && $y <= $my) {
      $mx = $x;
      $my = $y;
    }

    my $n = $path->xyxy_to_n_either($x,$y, $x+$dx,$y+$dy) // next N;
    $n < $n_hi or next N;
    if ($n >= $n_mid) {
      $above++;
    } else {
      $below++;
    }
  }
  next unless $above;
  next if $seen{"$mx,$my"}++;

  ($mx,$my) = transform($mx,$my);
  printwrap("($mx,$my),");
  $count++;
}
printwrap("}\n");

my $JA = JA($level);
$count == $JA or die "level=$level  count=$count JA=$JA";

print "    { \\p rectangle +(1,1) };\n";
print "  \\end{scope}\n";
print "% GP-Test  JA($level) == $JA\n";
print "  % end generated\n";
exit 0;
