#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90','rect_corners_sort',
  'BlobN','Lpred_k','Rpred';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 7;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

# end of the first bridge segment
my $n_hi = BlobN($level) - 3  + 1;

print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\node (start) at (0,0) [my dot];\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi+2);
  print "  \\coordinate (end) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi - 1);
  print "  \\node at ($x,$y) [my dot];\n";
}

my $PreBlobL = 0;
my $PreBlobR = 0;
my %seen;  # boundary squares
my %squares;

my $segments = MyLineSegments->new;
foreach my $n (0 .. $n_hi-1) {
  ### $n
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  $segments->add($x,$y, $x2,$y2);

  if (Lpred_k($level,$n)) {
    ### L ...
    $PreBlobL++;
    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
    my ($xc,$yc) = rect_corners_sort ($x,$y, $x2+$lx,$y2+$ly);
    if (! $seen{"$xc,$yc"}++) {
      push @{$squares{'L'}}, "$xc,$yc";
    }
  }
  if (Rpred($n)) {
    ### R ...
    $PreBlobR++;
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
    my ($xc,$yc) = rect_corners_sort ($x,$y, $x2+$rx,$y2+$ry);
    if (! $seen{"$xc,$yc"}++) {
      push @{$squares{'R'}}, "$xc,$yc";
    }
  }
}
foreach my $n ($n_hi .. $n_hi+1) {
  $segments->add($path->n_to_xy($n), $path->n_to_xy($n+1));
}

{
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

foreach my $side ('L','R') {
  my $aref = $squares{$side};
  $Printwrap = 2;
  printwrap("\\fill[my ${side}Q colour] foreach \\p in {");
  $Printwrap += 2;
  foreach my $xy (@$aref) {
    printwrap("($xy),");
  }
  printwrap("}\n");
  printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");
}

my $PreBlobLQ = scalar(@{$squares{'L'}});
my $PreBlobRQ = scalar(@{$squares{'R'}});
my $PreBlobBQ = $PreBlobLQ + $PreBlobRQ;
print "  \\newcommand{\\MyPreBlobLQ}{$PreBlobLQ}\n";
print "  \\newcommand{\\MyPreBlobRQ}{$PreBlobRQ}\n";
print "  \\newcommand{\\MyPreBlobBQ}{$PreBlobBQ}\n";
foreach my $side ('L','R') {
  print "% GP-Test-Last  PreBlob${side}Q($level) == ",
    scalar(@{$squares{$side}}),"\n";
}

print "  % end generated\n";
exit 0;
