#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Drawing downwards from the diameter, sub-trees directed left and down to
# make room for caption at right.

use 5.010;
use strict;
use warnings;
use Graph::Maker::TwindragonAreaTree;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'mul','chamfer','xy_rotate_minus90';
use MyGraphs 'Graph_subtree_depth';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $graph = Graph::Maker->new ('twindragon_area_tree',
                               level => $level,
                               undirected => 1);

my $segments = MyLineSegments->new;
my @vertices;
sub draw {
  my ($v, $prev, $x,$y, $dx,$dy) = @_;
  ### draw: $v

  push @vertices, "$x,$y";
  my @children = $graph->neighbours($v);
  @children = grep {$_ ne $prev} @children;
  ### @children
  ### num children: scalar(@children)
  die if @children > 2;

  @children = sort  # descending
    {Graph_subtree_depth($graph,$v,$b) <=> Graph_subtree_depth($graph,$v,$a)}
    @children;
  ### sorted: @children
  ### depths: [ map { Graph_subtree_depth($graph,$v,$_) } @children ]

  if (@children >= 1) {
    ### straight: $children[0]
    $segments->add($x,$y, $x+$dx,$y+$dy);
    draw($children[0], $v, $x+$dx,$y+$dy, $dx,$dy);
  }
  if (@children >= 2) {
    ### turn: $children[0]
    ($dx,$dy) = xy_rotate_minus90($dx,$dy);
    $segments->add($x,$y, $x+$dx,$y+$dy);
    draw($children[1], $v, $x+$dx,$y+$dy, $dx,$dy);
  }
}

draw('0', '', 0,0, 1,0);

$Printwrap = 2;
printwrap("\\draw[my tree] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\path foreach \\p in {");
$Printwrap = 4;
foreach my $xy (@vertices) {
  printwrap("($xy),");
}
printwrap("}");
printwrap(" { \\p node[my vertex] };\n");

my $diameter = $graph->diameter;
print "  \\newcommand{\\MyDiameter}{$diameter}\n";
print "% GP-Test-Last  TAdiameter($level) == $diameter\n";
print "  % end generated\n";
exit 0;
