#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()
use Math::NumSeq::PlanePathTurn;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'BlobN','BlobStart_xy','BlobEnd_xy','BlobDeltaLen',
  'xy_rotate_plus90','xy_rotate_minus90';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 12;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n\n";
print "% k = 5 to $max_level\n";
my $path = Math::PlanePath::DragonCurve->new;

sub xydxdy_to_enclosed_centre {
  my ($x,$y, $dx,$dy, $side, $n_limit) = @_;
  foreach (1 .. 3) {
    $x += $dx;
    $y += $dy;
    if ($side eq 'left') {
      ($dx,$dy) = xy_rotate_plus90($dx,$dy); # left
    } else {
      ($dx,$dy) = xy_rotate_minus90($dx,$dy); # right
    }
    my $n = $path->xyxy_to_n_either($x,$y, $x+$dx,$y+$dy);
    if (! defined $n || $n >= $n_limit) {
      return;
    }
  }
  my ($sx,$sy) =  ($side eq 'left'
                   ? xy_rotate_plus90($dx,$dy)
                   : xy_rotate_minus90($dx,$dy));
  return ($x + $dx/2 + $sx/2, $y + $dy/2 + $sy/2);
}

my @BlobAcl = (0);
my @BlobAcr = (1);
foreach my $k (5 .. $max_level) {
  my $n_blob_start = BlobN($k);
  my $n_blob_end = BlobN($k+1) - 3;

  my ($blob_start_x,$blob_start_y) = BlobStart_xy($k);
  my ($blob_end_x,$blob_end_y) = BlobEnd_xy($k);
  my $crossing_dx = $blob_end_x - $blob_start_x;
  my $crossing_dy = $blob_end_y - $blob_start_y;

  my %seen_square;
  my %count = (0 => 0,
               1 => 0,
               -1 => 0);
  foreach my $n ($n_blob_start .. $n_blob_end-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    foreach my $side ('left','right') {
      my ($mx,$my) = xydxdy_to_enclosed_centre($x,$y, $dx,$dy, $side, $n_blob_end)
        or next;
      next if $seen_square{"$mx,$my"}++;

      my $next_dx = $mx - $blob_end_x;
      my $next_dy = $my - $blob_end_y;

      # FIXME: this an undocumented func
      my $c = Math::NumSeq::PlanePathTurn::_turn_func_LSR
        ($crossing_dx,$crossing_dy, $next_dx,$next_dy,0);
      ### $c
      $count{$c}++;
    }
  }

  #$count{0} == BlobDeltaLen($k)+1 or die;

  my $BlobAcl = $count{1} + $count{0}/2;
  my $BlobAcr = $count{-1} + $count{0}/2;
  push @BlobAcl, $BlobAcl;
  push @BlobAcr, $BlobAcr;
}

sub to_frac {
  my ($n) = @_;
  return (int($n)==$n ? $n : (2*$n).'/2');
}

@BlobAcl = map {to_frac($_)} @BlobAcl;
@BlobAcr = map {to_frac($_)} @BlobAcr;

print "% GP-DEFINE  BlobAcl_generated_samples = \\\n";
print "% GP-DEFINE    [",join(',',@BlobAcl),"];\n";

print "% GP-DEFINE  BlobAcr_generated_samples = \\\n";
print "% GP-DEFINE    [",join(',',@BlobAcr),"];\n";

print "% end generated\n";
exit 0;
