#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyPrintwrap ':all';
$|=1;

my $k_limit = 16;

print "% generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

sub n_is_right_boundary {
  my ($n) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! defined $path->xyxy_to_n_either($x1,$y1, $x1s,$y1s)
          || ! defined $path->xyxy_to_n_either($x2,$y2, $x2s,$y2s)
          || ! defined $path->xyxy_to_n_either($x1s,$y1s, $x2s,$y2s));
}

sub n_is_left_boundary {
  my ($n) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to left side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! defined $path->xyxy_to_n_either($x1,$y1, $x1s,$y1s)
          || ! defined $path->xyxy_to_n_either($x2,$y2, $x2s,$y2s)
          || ! defined $path->xyxy_to_n_either($x1s,$y1s, $x2s,$y2s));
}


{
  print "% GP-DEFINE  BlobN_generated_samples = [ ";
  my $k = 4;
  my $prev_bridge = 1;
  my $join = '';
  for (my $n = 0; $k < $k_limit; $n++) {
    my $bridge = (n_is_left_boundary($n) && n_is_right_boundary($n));
    if ($prev_bridge && ! $bridge) {
      print "$join$n";
      $k++;
      $join = ',';
    }
    $prev_bridge = $bridge;
  }
  print " ];\n";
}
{
  print "% GP-DEFINE  BlobNend_generated_samples = [ ";
  my $k = 4;
  my $prev_bridge = 1;
  my $join = '';
  for (my $n = 0; $k < $k_limit; $n++) {
    my $bridge = (n_is_left_boundary($n) && n_is_right_boundary($n));
    if ($bridge && ! $prev_bridge) {
      print "$join$n";
      $k++;
      $join = ',';
    }
    $prev_bridge = $bridge;
  }
  print " ];\n";
}
print "% end generated\n";
exit 0;

