#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019, 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon '@dir4_to_dx','@dir4_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";

my $path = Math::PlanePath::ComplexMinus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $grid = MyLineSegments->new;

print "  \\begin{scope}[scale=.5]\n";

{
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];
      my $n_other = $path->xy_to_n($x+$dx,$y+$dy);
      $grid->add(2*$x+$dx - $dy,  2*$y+$dy - $dx,
                 2*$x+$dx + $dy,  2*$y+$dy + $dx);
    }
  }

  # outer grid
  {
    $Printwrap = 4;
    printwrap("\\draw ");
    $Printwrap += 2;
    $grid->printwrap;
    printwrap(";\n");
  }

}

printwrap_newline();
print "  \\end{scope}\n";

# points scaled by *2, so half for total length in grid
my $grid_length = $grid->length / 2;

# 4 sides each unit square, but non-boundary sides shared so subtract half
print "% GP-Test  4*2^$level - (4*2^$level - ComplexBoundary($level))/2 == $grid_length /* grid length */\n";

print "  % end generated\n";
exit 0;
