#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::ComplexPlus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon '@dir4_to_dx','@dir4_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::ComplexPlus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  \\begin{scope}[font=\\tiny]\n";

# eight squares
my $EightA = 0;
my @EightA_n_list;
{
  my $is_visited = sub {
    my ($x,$y) = @_;
    my $n = $path->xy_to_n($x,$y);
    return (defined $n && $n <= $n_hi);
  };

  $Printwrap = 4;
  printwrap("\\fill[my grey] foreach \\p in {");
  $Printwrap = 6;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    if ($is_visited->($x+1,$y)
        && $is_visited->($x-1,$y)
        && $is_visited->($x,$y+1)
        && $is_visited->($x,$y-1)

        && $is_visited->($x+1,$y+1)
        && $is_visited->($x+1,$y-1)
        && $is_visited->($x-1,$y+1)
        && $is_visited->($x-1,$y-1)
       ) {
      $x = 2*$x - 1;
      $y = 2*$y - 2;
      printwrap("($x,$y),");
      $EightA++;
      push @EightA_n_list, $n;
    }
  }
  printwrap("}");
  printwrap(" { \\p +(\\MyGap,\\MyGap) rectangle +(2-\\MyGap,2-\\MyGap) };\n");
}

{
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];
      my $n_other = $path->xy_to_n($x+$dx,$y+$dy);
      $segments->add(2*$x+$dx - $dy,  2*$y+$dy - $dx - 1,
                     2*$x+$dx + $dy,  2*$y+$dy + $dx - 1);
    }
  }

  {
    $Printwrap = 4;
    printwrap("\\draw ");
    $Printwrap += 2;
    $segments->printwrap;
    printwrap(";\n");
  }
}

{
  # numbers
  $Printwrap = 4;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    $x = 2*$x;
    $y = 2*$y - 1;
    printwrap(" \\node at ($x,$y) {$n};");
  }
  printwrap_newline();
}


printwrap_newline();
print "  \\end{scope}\n";

print "  \\newcommand{\\MyEightA}{$EightA}\n";
print "  \\newcommand{\\MyEightAN}{",join(',',@EightA_n_list),"}\n";
print "% GP-Test  EightA($level) == $EightA\n";
print "% GP-DEFINE  gen_complex_plus_eight_vector = [",join(',',@EightA_n_list),"];\n";
print "% GP-DEFINE  gen_complex_plus_eight_k = $level;\n";
print "  % end generated\n";
exit 0;
