#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::Geometry::Planar;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'hull_points_sort_by_angle',
  'P1_xy','P2_xy','P3_xy','P4_xy','P5_xy','P6_xy','P7_xy','P8_xy','P9_xy','P10_xy';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

my @levels = @ARGV;
unless (@levels) { @levels = (0,1,2,3); }
print "  % generated by tools/$FindBin::Script ",join(' ',@levels),"\n";

my $path = Math::PlanePath::DragonCurve->new;

my @P_funcs = (\&P1_xy,
               \&P2_xy,
               \&P3_xy,
               \&P4_xy,
               \&P5_xy,
               \&P6_xy,
               \&P7_xy,
               \&P8_xy,
               \&P9_xy,
               \&P10_xy);
sub xy_to_P_list {
  my ($k, $x,$y) = @_;
  my @ret;
  foreach my $i (0 .. $#P_funcs) {
    my ($fx,$fy) = $P_funcs[$i]->($k);
    if ($x == $fx && $y == $fy) {
      push @ret, 'P'.($i+1);
    }
  }
  return @ret;
}

foreach my $k (@levels) {

  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

  my $segments = MyLineSegments->new;
  my $points = [ [0,0] ];
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    $segments->add($x1,$y1, $x2,$y2);
    push @$points, [$x2,$y2];
  }
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$k)]\n");
  $Printwrap += 2;
  printwrap("\\draw[my grey] ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");

  if (@$points > 2) {
    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    if (@$points >= 5) {
      ### convexhull2 of: points_str($points)
      $planar = $planar->convexhull2;
      $points = $planar->points;
    }
    $points = hull_points_sort_by_angle($points, -$x_hi,-$y_hi);
  }

  $Printwrap -= 2;
  my $num = 1;
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    my @P_list = xy_to_P_list($k, $x,$y);
    my $name = join('-', @P_list);
    if ($name) {
      $name = " ($name-$k)";
    }
    printwrap("\\node$name at ($x,$y) [my dot];\n");

    foreach my $p (@P_list) {
      print "% GP-Test  $p($k) == $x + I*$y\n";
    }
    $num++;
  }

  printwrap("\\coordinate (start-$k) at (0,0);\n");
  printwrap("\\coordinate (end-$k) at ($x_hi,$y_hi);\n");
  print "  \\end{scope}\n";
}

print "  % end generated\n";
exit 0;
