#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'hull_expand','points_to_bigrat';
use MyPlanar;


my $max_level = 50;

print "% generated by tools/$FindBin::Script\n";

sub xy_to_expression {
  my ($x,$y) = @_;

  if ($x == 0 && $y == 0) {
    return '0';
  }
  my $str = '';
  if ($x) {
    $str = $x;
  }
  if ($y) {
    if ($y > 0 && length($str)) { $str .= '+'; }
    $str .= "$y*I";
  }
  return $str;
}

foreach my $squares (0,1) {
  my $var = $FindBin::Script;
  $var =~ s/\.pl$//;
  $var =~ tr/-/_/;
  if ($squares) { $var .= "_PQ"; }
  print "% GP-DEFINE  $var = {[\n";

  my $ex = 1;
  my $ey = 0;
  my $points;
  if (! $squares) {
    $points = [
               [0,0],
               [$ex,$ey],
              ];
  } else {
    #   --------
    #  |        |
    #  |        |
    #  0 ------ 1
    #  |        |
    #  |        |
    #   --------
    $points = [
               [0,-1],
               [1,-1],
               [1, 1],
               [0, 1],
              ];
  }

  my @areas;
  foreach my $k (0 .. $max_level) {
    print "% GP-DEFINE    /* $k */ [ ";
    my $join = '';
    foreach my $p (@$points) {
      my ($x,$y) = @$p;
      print $join, xy_to_expression($x,$y);
      $join = ',';
    }
    print " ]", ($k<$max_level ? "," : ""), "\n";
    push @areas, MyPlanar::points_to_area(points_to_bigrat($points));

    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
  }

  print "% GP-DEFINE  ]};\n";
  if (! $squares) {
    print "% GP-DEFINE  HA_samples_planar = [ ",join(', ',@areas)," ];\n";
  }
}

print "% end generated\n";
exit 0;
