#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  \\node (start) at (0,0) [my dot];\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform_to_horizontal($x,$y, $level);
  print "  \\coordinate (end) at ($x,$y) [my dot];\n";
}

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  ($x1,$y1) = transform_to_horizontal($x1,$y1, $level);
  ($x2,$y2) = transform_to_horizontal($x2,$y2, $level);
  $segments->add($x1,$y1, $x2,$y2);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");

print "  % end generated\n";
exit 0;
