#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'ffixed_u', 'ffixed_u_parent', 'ffixed_uDepth',
  'ffixed_row_w';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

my $max_row = 6;

print "  % generated by tools/$FindBin::Script\n";

sub u_to_binary_subtract {
  my ($u) = @_;
  ### u_to_binary_subtract(): $u
  $u % 4 == 0 or die;
  $u /= 4;
  if ($u == 0) { return "0"; }
  my $str = '';
  while ($u > 1) {
    ### at: "u=$u str=$str"
    my $d = $u & 1;
    $str = $d . $str;
    $u += $d;
    ($u & 1) == 0 or die;
    $u >>= 1;
  }
  if (length($str)) {
    $str = "-$str";
  }
  return "1$str";
}
sub binary_subtract_to_u {
  my ($str) = @_;
  if ($str =~ /-/) {
    my $sub = $str;
    $sub =~ s/^1-/0b/;
    $str =~ tr/1/0/;
    $str =~ s/^0-/0b1/ or die;
    ### $str
    ### $sub
    return 4*(oct($str) - oct($sub));
  } else {
    return 4*$str;
  }
}

$Printwrap = 2;
my %nodes;
my $max_m = ffixed_row_w($max_row);
foreach my $m (0 .. $max_m) {
  my $node = "u$m";
  $nodes{$node} = 1;
  my $u = ffixed_u($m);
  my $binstr = u_to_binary_subtract($u);
  binary_subtract_to_u($binstr) == $u or die "u=$u binstr=$binstr";
  my $str = $binstr;
  $str =~ s/-/{-}/;
  if ($m == 9) {
    $str = "\$u_{$m} = $str\$";
  } elsif ($m == 11) {
    $str = "\$$str \\mathrlap{{} = u_{$m}}\$";
  } elsif ($m <= 12) {
    $str = "\$\\mathllap{u_{$m}}{} = $str\$";
  } else {
    $str = "\$$str\$ \\\\[-.5ex] \$u_{$m}\$";
  }
  printwrap(" \\node ($node) at ($node) [my box] {$str};");
  printwrap_newline();
  my $negbin = u_to_binary_subtract($u);
  my @negbin = split /-?/, $binstr.'00';
  foreach my $i (1 .. $#negbin) { $negbin[$i] = -$negbin[$i]; }
  while (@negbin && $negbin[0]==0) { shift @negbin; }
  print "% GP-Test  negbin(ffixed_u($m)) == [",join(',',@negbin),"]\n";
}
printwrap_newline();

foreach my $m (2 .. $max_m) {
  my $parent = ffixed_u_parent($m);
  my $parent_node = "u$parent";
  exists $nodes{$parent_node} or die "oops, no parent $parent_node";
  printwrap(" \\draw ($parent_node) to (u$m);");
}
printwrap_newline();

print "  % end generated\n";
exit 0;
