#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_minus90','xy_rotate_plus90','mul',
  'transform_to_horizontal','BlobN','Rnext','Lnext_k','Rpred','Lpred_k';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


# must odd level so lines vertical/horizontal
my $level = 16;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  % A high-resolution approximation to dragon curve excluding,\n";
print "  % middle biggest blob, positioned for curve (0,0) to (1,0).\n";
print "  % Can fill or draw.\n";
print "  % Lines are drawn connecting the middles of boundary squares.\n";
print "  my non blob fractal pic/.pic={\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $scale = int(($level+2)/2);
my $extra_scale = 0;
if ($scale > 8) {
  $extra_scale = $scale - 8;
  $scale = 8;
}
print "  \\begin{scope}[scale=1/2^$scale]\n";

my $extra_scale_factor = 1/2**$extra_scale;
sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level+1);  # +1 due to mul()
  return ($x*$extra_scale_factor, $y*$extra_scale_factor);
}

my $segments = MyLineSegments->new;

# $side==1 for right, $side==-1 for left
sub n_to_cxcy {
  my ($n, $side) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;

  my $cx = ($x1+$x2 + $side*$dy)/2;
  my $cy = ($y1+$y2 - $side*$dx)/2;
  return mul($cx,$cy);
}

my $NonBlobRQ = 0;
my $NonBlobLQ = 0;

foreach my $range ([0, BlobN($level) - 3],
                   [BlobN($level+1) - 1, $n_hi-1]) {
  my ($n_start,$n_end) = @$range;
  Rpred($n_start) || die "oops";
  Lpred_k($level,$n_start) || die "oops";

  my ($x_start,$y_start) = mul($path->n_to_xy($n_start));
  my ($x_end,$y_end)     = mul($path->n_to_xy($n_end + 1));

  {
    my ($prev_cx,$prev_cy) = ($x_start,$y_start);
    for (my $n = $n_start; $n <= $n_end; $n = Rnext($n)) {
      my ($cx,$cy) = n_to_cxcy($n,1);
      ### $n
      ### cxcy: "$cx,$cy"

      $NonBlobRQ += ($cx != $prev_cx || $cy != $prev_cy);
      $segments->add($prev_cx,$prev_cy, $cx,$cy);
      $prev_cx = $cx;
      $prev_cy = $cy;
    }
    $segments->add($prev_cx,$prev_cy, $x_end,$y_end);
  }
  {
    my ($prev_cx,$prev_cy) = ($x_start,$y_start);
    for (my $n = $n_start; defined $n && $n<=$n_end; $n = Lnext_k($level,$n)) {
      defined $n || die;
      my ($cx,$cy) = n_to_cxcy($n,-1);
      ### cxcy: "$cx,$cy"

      $NonBlobLQ += ($cx != $prev_cx || $cy != $prev_cy);
      $segments->add($prev_cx,$prev_cy, $cx,$cy);
      $prev_cx = $cx;
      $prev_cy = $cy;
    }
    $segments->add($prev_cx,$prev_cy, $x_end,$y_end);
  }
}

$Printwrap = 2;
printwrap("\\path[pic actions] ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");
my $draw_count = $segments->lines_joined_count_points;


print "  \\end{scope}\n";
print "% GP-Test-Last  RQ($level)-BlobRQ($level) == $NonBlobRQ\n";
print "% GP-Test-Last  LQ($level)-BlobLQ($level) == $NonBlobLQ\n";
print "  % drawn with $draw_count points\n";
print "  },\n";
print "  % end generated\n";
exit 0;
