#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90','BlobStart_xy','BlobEnd_xy',
  'transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
### level: "$level  n_hi=$n_hi"

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level+1);
}

my $y_min;
my $y_max;
my $x_min;
my $x_max;

{
  my ($x,$y) = transform(0,0);
  print "  \\node (start) at ($x,$y) [my dot];\n";
  $x_min = $x - 5;
}
{
  my ($x,$y) = transform($x_hi,$y_hi);
  print "  \\node (end) at ($x,$y) [my dot];\n";
  $x_max = $x + 5;
}

{
  my ($blob_x,$blob_y) = BlobStart_xy($level-1);

  my ($x_middle_start,$y_middle_start)
    = xy_rotate_plus90($blob_x,$blob_y);

  my ($x_middle_end, $y_middle_end)
    = ($x_hi-$x_middle_start, $y_hi-$y_middle_start);

  print "% GP-Test  BlobStart($level-1) == $blob_x + I*$blob_y\n";
  print "% GP-Test  TMiddle_start($level) == $x_middle_start + I*$y_middle_start\n";
  print "% GP-Test  TMiddle_end($level) == $x_middle_end + I*$y_middle_end\n";

  my ($sx,$sy) = transform($x_middle_start, $y_middle_start);
  my ($tx,$ty) = transform($x_middle_end, $y_middle_end);
  print "  \\coordinate (middle-start) at  ($sx,$sy);\n";
  print "  \\coordinate (middle-end) at  ($tx,$ty);\n";

  $y_min = $sy - 3;
  $y_max = $ty + 3;

  my $dx = $tx - $sx;
  my $dy = $ty - $sy;
  my $sqrt2 = '';
  if ($level % 2 == 0) {
    $dx /= 2;
    $dy /= 2;
    $sqrt2 = '*sqrt2';
  }
  print "% GP-Test  TMiddle_x($level) == $dx$sqrt2\n";
  print "% GP-Test  TMiddle_y($level) == $dy$sqrt2\n";
}

sub clip {
  my ($x1,$y1, $x2,$y2) = @_;
  ($x1,$y1) = transform($x1,$y1);
  ($x2,$y2) = transform($x2,$y2);
  return ($x_min <= $x1 && $x1 <= $x_max
          && $y_min <= $y1 && $y1 <= $y_max
          && $x_min <= $x2 && $x2 <= $x_max
          && $y_min <= $y2 && $y2 <= $y_max);
}

my $segments = MyLineSegments->new;

foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);

  if (clip($x,$y, $x2,$y2)) {
    $segments->add($x,$y, $x2,$y2);
  }

  ($x,$y) = ($x_hi-$x, $y_hi-$y);
  ($x2,$y2) = ($x_hi-$x2, $y_hi-$y2);

  if (clip($x,$y, $x2,$y2)) {
    $segments->add($x,$y, $x2,$y2);
  }
}

$Printwrap = 2;
printwrap("\\draw[my dragon] ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw[my line]");
$Printwrap += 2;
foreach my $delta (2 .. 3) {
  my ($sx,$sy) = BlobStart_xy($level - $delta);
  my ($tx,$ty) = BlobEnd_xy($level - $delta);
  ($sx,$sy) = xy_rotate_plus90($sx,$sy);
  ($tx,$ty) = xy_rotate_plus90($tx,$ty);
  {
    my ($sx,$sy) = transform($sx,$sy);
    my ($tx,$ty) = transform($tx,$ty);
    printwrap(" ($sx,$sy)--($tx,$ty)");
  }

  ($sx,$sy) = ($x_hi-$sx, $y_hi-$sy);
  ($tx,$ty) = ($x_hi-$tx, $y_hi-$ty);
  ($sx,$sy) = transform($sx,$sy);
  ($tx,$ty) = transform($tx,$ty);
  printwrap(" ($sx,$sy)--($tx,$ty)");
}
printwrap(";\n");

print "  % end generated\n";
exit 0;
