#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::Geometry::Planar;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'points_to_hull','xy_to_node_name';
use MyPrintwrap ':all';
use MyPlanar;

use Math::PlanePath::GcdRationals;
BEGIN { *gcd = \&Math::PlanePath::GcdRationals::_gcd; }

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 4;  # twindragon level

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($ex,$ey) = $path->n_to_xy($n_hi);

print "  \\coordinate (start) at (0,0);\n";
print "  \\coordinate (end) at ($ex,$ey);\n";

my @segments;
my $points = [];
{
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    foreach my $rev (0,1) {
      push @$points, [$x1,$y1];
      push @segments, [$x1,$y1, $x2,$y2];
      $x1 = $ex - $x1;
      $y1 = $ey - $y1;
      $x2 = $ex - $x2;
      $y2 = $ey - $y2;
    }
  }
}

my $planar = Math::Geometry::Planar->new;
$planar->points($points);
if (@$points >= 5) {
  $planar = $planar->convexhull2;
  $points = $planar->points;
}

my %boundary_points;
$Printwrap = 2;
foreach my $i (0 .. $#$points) {
  my $p1 = $points->[$i];
  my $p2 = $points->[($i+1) % scalar(@$points)];
  my ($x,$y) = @$p1;
  my ($x2,$y2) = @$p2;

  my $P = $i+1;
  $boundary_points{"$x,$y"} = 1;

  my $dx = $x2 - $x;
  my $dy = $y2 - $y;
  my $g = gcd($dx,$dy);
  $dx /= $g;
  $dy /= $g;

  $x += $dx;
  $y += $dy;
  while ($x != $x2 || $y != $y2) {
    $boundary_points{"$x,$y"} = 1;
    $x += $dx;
    $y += $dy;
  }
}
printwrap_newline();
### %boundary_points
my $THBP = scalar keys %boundary_points;

$Printwrap = 2;
foreach my $p (sort keys %boundary_points) {
  my ($x,$y) = split /,/, $p;
  my $name = xy_to_node_name($x,$y);
  printwrap(" \\node ($name) at ($x,$y) [my boundary dot];");
}
printwrap_newline();
print "  \\newcommand{\\MyTHBP}{$THBP}\n";
print "% GP-Test-Last  THBP($level)==$THBP\n";

# {
#   $Printwrap = 2;
#   printwrap("\\draw[my hull line]\n");
#   $Printwrap = 4;
#   my $join = '';
#   foreach my $p (@$points) {
#     my ($x,$y) = @$p;
#     printwrap("$join($x,$y)");
#     $join = '--';
#   }
#   printwrap("${join}cycle;\n");
# }

my $THIP = 0;
$Printwrap = 2;
my ($xmin,$ymin, $xmax,$ymax) = $planar->MyPlanar::bbox_xyxy;
foreach my $x ($xmin .. $xmax) {
  foreach my $y ($ymin .. $ymax) {
    if ($planar->isinside([$x,$y])) {
      my $p = "$x,$y";
      next if $boundary_points{$p};
      my $name = xy_to_node_name($x,$y);
      printwrap(" \\node ($name) at ($x,$y) [my inside dot];");
      $THIP++;
    }
  }
}
printwrap_newline();
print "  \\newcommand{\\MyTHIP}{$THIP}\n";
print "% GP-Test-Last  THIP($level)==$THIP\n";

$Printwrap = 2;
printwrap("\\draw[my twindragon line]");
$Printwrap = 4;
foreach my $seg (@segments) {
  my ($x1,$y1, $x2,$y2) = @$seg;
  my $name1 = xy_to_node_name($x1,$y1);
  my $name2 = xy_to_node_name($x2,$y2);
  printwrap(" ($name1)--($name2)");
}
printwrap(";\n");

# {
#   my $i = 1;
#   foreach my $p (@$points) {
#     my ($x,$y) = @$p;
#     print "% GP-Test   T_P$i($level) == $x+I*$y\n";
#     $i++;
#   }
# }

my $THP = $THBP + $THIP;
print "  \\newcommand{\\MyTHP}{$THP}\n";
print "% GP-Test-Last  THP($level)==$THP\n";

print "  % end generated\n";
exit 0;
