#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'BlobN','Bpred_k_flat',
  'transform_to_horizontal','xy_rotate_minus90';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 15;

print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo,$n_hi) = $path->level_to_n_range($level);
my $scale = 2**(($level+1)/2 + 1);
print "  \\begin{scope}[scale=1/$scale]\n";

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level+1);
  $x *= 2;
  $y *= 2;
  return ($x,$y);
}

my $segments = MyLineSegments->new;
{
  my $n_start = BlobN($level+1);
  my $count = 0;
  foreach my $n ($n_start .. $n_hi-1) {
    my $n2 = sprintf '%0*b', $level, $n;
    if (Bpred_k_flat()->contains($n2)) {
      $count++;
      $segments->add($path->n_to_xy($n),
                     $path->n_to_xy($n+1));
    }
  }
  print "  % first half $n_start to $n_hi, count $count\n";
}
{
  my $segments2 = MyLineSegments->new;
  my ($x_hi,$y_hi) = $path->n_to_xy(2*$n_hi);
  my $n_start = BlobN($level);
  my $count = 0;
  foreach my $n ($n_start .. $n_hi-1) {
    my $n2 = sprintf '%0*b', $level, $n;
    if (Bpred_k_flat()->contains($n2)) {
      $count++;

      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      ($x1,$y1) = xy_rotate_minus90($x1,$y1);
      ($x2,$y2) = xy_rotate_minus90($x2,$y2);
      $x1 += $x_hi; $y1 += $y_hi;
      $x2 += $x_hi; $y2 += $y_hi;
      $segments->add($x1,$y1, $x2,$y2);
    }
  }
  print "  % second half $n_start to $n_hi, count $count\n";
}


$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap (transform => sub {
                        my ($x,$y) = @_;
                        ($x,$y) = transform($x,$y);
                        return ($x,$y);
                      });
printwrap(";");
printwrap(" % drawn with " . ($segments->lines_joined_count_points) . " points\n");

print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
