#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::Complex;

BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 12;

sub i_offset {
  my ($k, $pos) = @_;
  my $z = 0;
  for (my $i = $pos; $i < $k; $i += 2) {
    $z += Math::Complex->make(-1, 1) ** $i;
  }
  return $z;
}

for my $pos (1, 0) {
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

{
    print "  \\begin{scope}[shift=(pos-$pos)]\n";
  my $prev;
  foreach my $k (0 .. $level) {
    my $z = i_offset($k,$pos);
    if (defined $prev) {
      if ($
    unless (defined $prev && $prev == $z) {
    printwrap(" \\node at ($x,$y) [my dot] {};");
    $prev = $z;
  }
  printwrap_newline();
}

{
  $Printwrap = 3;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];
      my $n_other = $path->xy_to_n($x+$dx,$y+$dy);
      my $segs = (defined $n_other && $n_other <= $n_hi
                  ? $insides     # inside side
                  : $boundary);  # boundary side
      $segs->add(2*$x+$dx - $dy,  2*$y+$dy - $dx - 1,
                 2*$x+$dx + $dy,  2*$y+$dy + $dx - 1);
    }
  }

  $Printwrap = 4;
  printwrap("\\draw");
  $Printwrap = 5;
  foreach my $line ($boundary->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;
    printwrap(" ($x1,$y1)--($x2,$y2)");
  }
  printwrap(";\n");

  $Printwrap = 4;
  printwrap("\\draw[dotted]");
  $Printwrap = 5;
  foreach my $line ($insides->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;
    printwrap(" ($x1,$y1)--($x2,$y2)");
  }
  printwrap(";\n");

  # inside corners
  {
    my $is_visited = sub {
      my ($x,$y) = @_;
      my $n = $path->xy_to_n($x,$y);
      return (defined $n && $n <= $n_hi);
    };
    $Printwrap = 3;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      if ($is_visited->($x+1,$y)
          && $is_visited->($x+1,$y+1)
          && $is_visited->($x,$y+1)) {
        $x = 2*$x + 1;
        $y = 2*$y;
        printwrap(" \\node at ($x,$y) [my dot] {};");
      }
    }
    printwrap_newline();
  }
}

printwrap_newline();
print "  \\end{scope}\n";

# /2 because 2*X,2*Y coordinates
my $boundary_length = $boundary->length / 2;
my $insides_length = $insides->length / 2;
print "  % boundary length $boundary_length insides $insides_length\n";
print "  % GP-Test  TS($level) == $boundary_length\n";
print "  % GP-Test  TD($level) == $insides_length\n";

print "  % end generated\n";
exit 0;
