#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'ffixed_du','ffixed_du_parent','ffixed_row_w';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
use Smart::Comments;

my $max_row = 7;

print "  % generated by tools/$FindBin::Script\n";

my $max_m = ffixed_row_w($max_row) - 1;
my %node_to_xy = (du0 => "-1,0");
my $bottom_x = 0;
### $bottom_x
foreach my $m (ffixed_row_w($max_row-1) .. $max_m) {
  my $node = "du$m";
  my $y = -$max_row + 1;
  $node_to_xy{$node} = "$bottom_x,$y";
  $bottom_x++;
}

foreach my $m (reverse 1 .. $max_m) {
  my $node = "du$m";
  my $parent = ffixed_du_parent($m) // next;
  ### above: "parent=$parent"
  my $parent_node = "du$parent";
  exists $node_to_xy{$node} or die "oops, no $node";
  my ($x,$y) = split /,/, $node_to_xy{$node};
  $y++;
  $node_to_xy{$parent_node} = "$x,$y";
}

$Printwrap = 2;
foreach my $m (0 .. $max_m) {
  my $node = "du$m";
  my ($x,$y) = split /,/, $node_to_xy{$node};
  my $du = ffixed_du($m);
  printwrap(" \\node ($node) at ($x,$y) [my box] {\$$du\$};");
}
printwrap_newline();

foreach my $m (2 .. $max_m) {
  my $parent = ffixed_du_parent($m) // next;
  my $parent_node = "du$parent";
  exists $node_to_xy{$parent_node} or die "oops, no parent $parent_node";
  printwrap(" \\draw ($parent_node) to (du$m);");
}
printwrap_newline();

print "% GP-Test-Last  vector(",$max_m+1,",m,m--; ffixed_du(m)) == \\\n";
print "% GP-Test-Last    [",join(',',map {ffixed_du($_)} 0 .. $max_m),"]\n";

print "  % end generated\n";
exit 0;
